#
#       HBS REDUCTION SUB ROUTINE
#
#      Prescan and prefit for template images (wavelength derection)
#
#       For no scan data group (2 hole data only)
#
#                                                 : hbs1to1x2.cl
#
#
#      The template images must be ordered by star position from
#      UPPER to LOWER diaph.
#
#
#

procedure hbs1to1x2

begin

  file fn_root
  file fn_rootnum
  file fn_scanp, fn_scanw
  string fn_fitp[2], fn_fitw[2], fn_tmp
  string tmp_tbl2[2]
  file fn_scan, infn
  file fn_y2 = "hbs1to1fitx.dat"
  file tmp_fn, tmp_list, tmp_tbl1
  int i, j, k, l, m
  int nx, ny
  real dx = 10
  real y[2], y1, y2, y3
  int n_image = 20
  real c[6], c1[6], c2[6]
  int n_data = 1  # The extetion will be 00, 01,..., nn
  int n_eq1 = 3
  int n_eq2 = 15

  string fn_stack[300]
  string hbslist[50]
  string shorttyp[50], title[50]
  string exptime[50], diaph[50], filter[50]
  string slit[50]
  string listoflist
  string buf = ""
  string buf1, buf2, buf3, buf4, buf5, buf6, buf7
  int n_list

#----- initial settings

  listoflist = envget( "_rootlist" )
  fn_fitp[1] = envget( "_1to1we" )
  fn_fitp[2] = envget( "_1to1wo" )


  tmp_fn = mktemp( "hbs.tmp.image." )
  tmp_list = mktemp( "hbs.tmp.list." )
  tmp_tbl1 = mktemp( "hbs.tmp.tbl." )
  tmp_tbl2[1] = mktemp( "hbs.tmp.tbl." )
  tmp_tbl2[2] = mktemp( "hbs.tmp.tbl." )

  print( "" )
  print( "-------------------------------------" )
  print( " X-direction Fit for template images" )
  print( "-------------------------------------" )
  print( "" )


  if( access( listoflist ) == 0 )
  {
    print( "Cannot find "//listoflist//"   Please do hbsgroup" )
    bye
  }

  for( i=1; i<=2; i+=1 )
  {
    fn_fitw[i] = fn_root//"width."//i//".tbl"
    if( access( fn_fitp[i] ) == 1 )
      delete( fn_fitp[i], ver- )
    if( access( fn_fitw[i] ) == 1 )
      delete( fn_fitw[i], ver- )
  }

  if( access( tmp_tbl2[1] ) == 1 )
    delete( tmp_tbl2[1], ver- )
  if( access( tmp_tbl2[2] ) == 1 )
    delete( tmp_tbl2[2], ver- )

#----- Scanning the list file

  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "HBS" )
    {
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL" )
      break
    }
    if( buf2 == "DAR" || buf2 == "BIA" || buf2 == "NEA" )
      next

    n_list += 1
    hbslist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#----- make averaged images

  if( access( tmp_list ) == 1 )
    delete( tmp_list, ver- )

  printf( "-----\n" )
  for( i=1; i<=n_list; i+=1 )
    printf( "%2d %s %s %s\n", i, hbslist[i], title[i], diaph[i] )
  printf( "-----\n" )
  printf( "The lists must be ordered by star positions from UPPER to LOWER diaph\n" )

  printf( "Input head name of averaged image >> " )
  l = scan( fn_root )

  for( k=0; k<=n_data; k+=1 )
  {
    buf = "00"//str(k)
    fn_rootnum = fn_root//substr( buf, ( strlen(buf) - 1 ), strlen( buf) )//"."
    fn_scan = fn_rootnum//"tr"
    if( access( fn_scan//".imh" ) == 1 )
      imdelete( fn_scan, ver- )

    printf( "Select for %d/%dth group >> ", k, n_data )

    l = scan( m )
    while( m < 1 || m > n_list )
      l = scan( m )
#    sed( 's/fit\.imh/tr/', hbslist[m], >> tmp_list )
    sed( 's/fit\.imh/px/', hbslist[m], >> tmp_list )
    imsum( "@"//tmp_list, fn_scan, hparam="EXPTIME", pixtype="real",\
           calctype="real", option="sum", ver- )
    delete( tmp_list, ver- )
  }

#---- main routine

  for( k=0; k<=n_data; k+=1 )
  {
    buf = "00"//str(k)
    fn_rootnum = fn_root//substr( buf, ( strlen(buf) - 1 ), strlen( buf) )//"."
    fn_scan = fn_rootnum//"tr"
    fn_scanp = fn_rootnum//"pek."
    fn_scanw = fn_rootnum//"wid."

    if( access( fn_scan//".imh" ) == 0 )
    {
      printf( "Cannot find '"//fn_scan//"'. " )
      exit( -1 )
    }

prescanning:

    boxcar( fn_scan, tmp_fn, dx, 1, boundary="nearest" )

#---- search four spectra and define initial constants

    if( access( "database/id"//fn_scan ) )
        delete( "database/id"//fn_scan, ver- )
    if( access( fn_y2 ) )
        delete( fn_y2, ver- )

    print( "mark the 1st and 2nd peaks in order with [m]-key" )

    identify( fn_scan, section="middle column", databas='database',\
              coordli='home$etc/null.dat', nsum=500, match=20.,\
              maxfeat=2, fwidth=15., cradius=15., functio='legendre',\
              order=2, niterate=10, low_rej=3., high_rej=3.,\
              grow=0.,ftype='emission', mode='ql',thresho=5,\
              autowrite+ )

    list = "database/id"//fn_scan

    for( i=1; i<=2; i+=1 )
    {
      while( fscan( list, buf1, buf, buf, buf, buf, buf ) != 6 ){}
      y[i] = real( buf1 )
#      printf( "%6.2f\n", y[i], >> fn_y2 )
    }
    delete( "database/id"//fn_scan )

#---- scan two spectra

    imgets( fn_scan, 'i_naxis1' )
    nx = int( imgets.value )
    imgets( fn_scan, 'i_naxis2' )
    ny = int( imgets.value )

    if( access( tmp_tbl1 ) == 1 )
      delete( tmp_tbl1, ver- )

    for( j=1; j<=2; j+=1 )
    {
      if( access( fn_scanp//j ) == 1 )
        delete( fn_scanp//j, ver- )
#      if( access( fn_scanw//j ) == 1 )
#        delete( fn_scanw//j, ver- )
    }

    wfits( tmp_fn, tmp_tbl1, bscale=1.0, bzero=0.0, long_header-,
           short_header-, bitpix=-32, scale-, autoscale- )

    hbsscan2p( tmp_tbl1, y[1], y[2], 80, nx, fn_scanp )
#    hbsscan2w( tmp_tbl1, y[1], y[2], 0.05, 80, nx, fn_scanw )

    delete( tmp_tbl1, ver- )

    imdelete( tmp_fn, ver- )


#---- search plot files

    for( i=1; i<=2; i+=1 )
    {
      infn = fn_scanp//i
      if( access( infn ) == 0 )
      {
        print( "cannnot find "//infn )
        bye
      }
#      infn = fn_scanw//i
#      if( access( infn ) == 0 )
#      {
#        print( "cannnot find "//infn )
#        bye
#      }
    }
 
#----- Scanning the list file

    for( i=1; i<=2; i+=1 )
    {

    #---- peak ----
      infn = fn_scanp//i

      curfit( infn, function="legendre", weighting="uniform",\
              order=6, interactive+, axis=1, listdata-, ver-,\
              calctype="double", power+, device="stdgraph", cursor="",\
              > tmp_tbl1 )

      list = tmp_tbl1

      while( fscan( list, buf, buf1, buf2, buf3, buf4, buf5,\
                    buf6, buf7 ) != EOF )
      {
        if( strlen( buf ) != 1 )
          next
        j = int( buf )
        if( j > 0 )
        {
          c[j] = real( buf1 )
        }
      }

#      fn_tmp = fn_fitp[i]
      fn_tmp = tmp_tbl2[i]
      printf( "%12.9e %12.9e %12.9e %12.9e %12.9e %12.9e\n",\
              c[1], c[2], c[3], c[4], c[5], c[6], >> fn_tmp )

      delete( tmp_tbl1, ver- )

    #---- width ----
#      infn = fn_scanw//i

#      curfit( infn, function="legendre", weighting="uniform",\
#              order=6, interactive+, axis=1, listdata-, ver-,\
#              calctype="double", power+, device="stdgraph", cursor="",\
#              > tmp_tbl1 )

#      list = tmp_tbl1

#      while( fscan( list, buf, buf1, buf2, buf3, buf4, buf5,\
#                    buf6, buf7 ) != EOF )
#      {
#        if( strlen( buf ) != 1 )
#          next
#        j = int( buf )
#        if( j > 0 )
#        {
#          c[j] = real( buf1 )
#        }
#      }

#      fn_tmp = fn_fitw[i]
#      printf( "%12.9e %12.9e %12.9e %12.9e %12.9e %12.9e\n",\
#              c[1], c[2], c[3], c[4], c[5], c[6], >> fn_tmp )

#      delete( tmp_tbl1, ver- )
    }
  }

#---- scaling 2 data --> 21 data

  for( i=1; i<=2; i+=1 )
  {

    list = tmp_tbl2[i]

    for( j=1; j<=2; j+=1 )
    {
      if(  fscan( list, buf1, buf2, buf3, buf4, buf5,\
                  buf6 ) == EOF )
      {
        printf( "Read error in scaling '%s'\n", tmp_tbl2[i] )
	bye
      }
      if( j == 1 )
      {
        c1[1] = real( buf1 )
        c1[2] = real( buf2 )
        c1[3] = real( buf3 )
        c1[4] = real( buf4 )
        c1[5] = real( buf5 )
        c1[6] = real( buf6 )
      }
      else
      {
        c2[1] = real( buf1 )
        c2[2] = real( buf2 )
        c2[3] = real( buf3 )
        c2[4] = real( buf4 )
        c2[5] = real( buf5 )
        c2[6] = real( buf6 )
      }
    }
    delete( tmp_tbl2[i], ver- )

    for( j=0; j<=n_image; j+=1 )
    {
      for( k=1; k<=6; k+=1 )
      {
        c[k] = c1[k] + ( c2[k] - c1[k] ) / ( n_eq2 - n_eq1 ) * ( j - n_eq1 )
      }

      fn_tmp = fn_fitp[i]
      printf( "%12.9e %12.9e %12.9e %12.9e %12.9e %12.9e\n",\
              c[1], c[2], c[3], c[4], c[5], c[6], >> fn_tmp )
    }
  }
  
end
