#
#       HBS REDUCTION SUB ROUTINE
#
#      Prescan and prefit for template images (slit-length derection)
#       For 2HL diaph data only and no line longward from 7000A : hbs1to1y3.cl
#
#


procedure hbs1to1y3

begin

  file fn_root = "Slit_hct."
  string fn_fit[2]
  file fn_fit_tmp
  string fn_scan0[2] # original *.tr image
  string fn_scan1[2] # y-scaled *.tr image
  string fn_scan2[2] # transposed image
  file fn_y2 = "hbs1to1fity.dat"
  file infn
  file tmp_fn, tmp_list, tmp_tbl
  int i, j, k, l, m
  int nx, ny
  real slicelevel = 0.1
  real dx = 10
  real y[40], lmd[40]
  int n_feature = 10
  real c[6]
  string caldir

  string fn_stack[300]
  string hbslist[50]
  string shorttyp[50], title[50]
  string exptime[50], diaph[50], filter[50]
  string slit[50]
  string listoflist
  string buf = ""
  string buf1, buf2, buf3, buf4, buf5, buf6, buf7
  int n_list
  int n_version      # IRAF version (210 or 211)

#----- initial settings

  caldir = envget( "_caldir" )
  fn_fit[1] = envget( "_1to1se" )
  fn_fit[2] = envget( "_1to1so" )
  fn_y2 = envget( "_1to1sw" )
  listoflist = envget( "_rootlist" )

  fn_scan0[1] = fn_root//"e.tr"
  fn_scan0[2] = fn_root//"o.tr"
  fn_scan1[1] = fn_root//"e.t1"
  fn_scan1[2] = fn_root//"o.t1"
  fn_scan2[1] = fn_root//"e.tp"
  fn_scan2[2] = fn_root//"o.tp"

  for( i=1; i<=2; i+=1 )
  {
    if( access( fn_scan0[i]//".imh" ) == 1 )
      imdelete( fn_scan0[i], ver- )
    if( access( fn_scan1[i]//".imh" ) == 1 )
      imdelete( fn_scan1[i], ver- )
    if( access( fn_scan2[i]//".imh" ) == 1 )
      imdelete( fn_scan2[i], ver- )
  }

  tmp_list = mktemp( "hbs.tmp.list." )
  tmp_tbl = mktemp( "hbs.tmp.tbl." )
  tmp_fn = mktemp( "hbs.tmp.image." )

  print( "" )
  print( "-------------------------------------------" )
  print( " Y-direction Fit for slit-length direction" )
  print( "-------------------------------------------" )
  print( "" )

#----- IRAF version check

  buf1 = envget( "version" )
  i = stridx( "V", buf1 )
  buf2 = substr( buf1, i, ( i + 4 ) )

  n_version = 0

  if( buf2 == "V2.11" )
  {
    n_version = 211
  }
  else
  {
    if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'hbsred'\n"
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n"
    bye
  }

#---

  for( i=1; i<=2; i+=1 )
  {
    if( access( fn_scan1[i]//".imh" ) == 1 )
      imdelete( fn_scan1[i], ver- )
    if( access( fn_scan2[i]//".imh" ) == 1 )
      imdelete( fn_scan2[i], ver- )
    if( access( fn_fit[i] ) == 1 )
      delete( fn_fit[i], ver- )
  }

#----- Scanning the list file

  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "HBS" )
    {
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL" )
      break
    }
    if( buf2 == "DAR" || buf2 == "BIA" || buf2 == "NEA" )
      next

    n_list += 1
    hbslist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }


#----- make averaged images

  if( access( tmp_list ) == 1 )
    delete( tmp_list, ver- )

  printf( "-----\n" )
  for( i=1; i<=n_list; i+=1 )
    printf( "%2d %s %s %s\n", i, hbslist[i], title[i], filter[i] )
  printf( "-----\n" )

  for( k=1; k<=2; k+=1 )
  {
    if( k == 1 )
      printf( "Select for group of extraordinary ray >> " )
    else
      printf( "Select for group of ordinary ray >> " )

    l = scan( m )
    while( m < 1 || m > n_list )
      l = scan( m )
    sed( 's/fit\.imh/px/', hbslist[m], >> tmp_list )
#    sed( 's/fit\.imh/tr/', hbslist[m], >> tmp_list )
    imsum( "@"//tmp_list, fn_scan0[k], hparam="EXPTIME", pixtype="real",\
           calctype="real", option="sum", ver- )
    boxcar( fn_scan0[k], fn_scan0[k], 1, 21 )
    delete( tmp_list, ver- )
  }

#---- make transposed image

  for( k=1; k<=2; k+=1 )
  {

    if( k == 1 )
      hbsscy( fn_scan0[k], caldir, fn_scan1[k], tmp_fn )
    else
      hbsscy( fn_scan0[k], caldir, tmp_fn, fn_scan1[k] )

    imdelete( tmp_fn, ver- )

    imtranspose( fn_scan1[k], fn_scan2[k] )

prescanning:

    boxcar( fn_scan2[k], tmp_fn, dx, 1, boundary="nearest" )

    if( k == 1 )
    {

#---- identify 25 lines and define initial constants

      if( access( "database/id"//fn_scan2[k] ) )
        delete( "database/id"//fn_scan2[k], ver- )
      if( access( fn_y2 ) )
        delete( fn_y2, ver- )

      type( "home$etc/Hg-Ne.dat" )

      print( "mark the emission lines in order with [m]-key" )

      identify( fn_scan2[k], section="middle column", databas='database',\
                coordli='home$etc/Hg-Ne.dat', nsum=10, match=20.,\
                maxfeat=n_feature, fwidth=5., cradius=10., functio='legendre',\
                order=4, niterate=10, low_rej=3., high_rej=3.,\
                grow=0.,ftype='emission', mode='ql',thresho=5,\
                autowrite+ )

      list = "database/id"//fn_scan2[k]

      for( i=1; i<=n_feature; i+=1 )
      {
        while( fscan( list, buf1, buf, buf2, buf, buf, buf ) != 6 ){}
        y[i] = real( buf1 )
        lmd[i] = real( buf2 )
        printf( "%6.2f  %9.4f\n", y[i], lmd[i], >> fn_y2 )
      }
    }

#---- scan n_feature spectra

    imgets( fn_scan2[k], 'i_naxis1' )
    nx = int( imgets.value )
    imgets( fn_scan2[k], 'i_naxis2' )
    ny = int( imgets.value )

    if( access( tmp_tbl ) == 1 )
      delete( tmp_tbl, ver- )

    if( n_version == 210 )
    {
      wfits( tmp_fn, tmp_tbl, bscale=1.0, bzero=0.0, long_header-,
             short_header-, bitpix=-32, scale-, autoscale- )
    }
    else
    {
      if( n_version == 211 )
      {
        wfits( tmp_fn, tmp_tbl, bscale=1.0, bzero=0.0, long_header-,
               fextn="", short_header-, bitpix=-32, scale-, autoscale- )
      }
      else
      {
        printf( "IRAF version erorr (%d)\n", n_version )
        bye
      }
    }

    for( j=1; j<=n_feature; j+=1 )
    {
      printf( "%2d ", j )
      infn = fn_root//j
      if( access( infn ) == 1 )
        delete( infn, ver- )
      hbsscan1p( tmp_tbl, y[j], 40, (nx-40), infn )
    }
    printf( "\n" )

    delete( tmp_tbl, ver- )

    imdelete( tmp_fn, ver- )
    imdelete( fn_scan2[k], ver- )

#---- search plot files

    for( i=1; i<=n_feature; i+=1 )
    {
      infn = fn_root//i
      if( access( infn ) == 0 )
      {
        print( "cannnot find "//infn )
        bye
      }
    }
 
#----- Scanning the list file

    for( i=1; i<=n_feature; i+=1 )
    {

      infn = fn_root//i

      curfit( infn, function="legendre", weighting="uniform",\
              order=4, interactive+, axis=1, listdata-, ver-,\
              calctype="double", power+, device="stdgraph", cursor="",\
              > tmp_tbl )

      list = tmp_tbl

      while( fscan( list, buf, buf1, buf2, buf3, buf4, buf5,\
                    buf6, buf7 ) != EOF )
      {
        if( strlen( buf ) != 1 )
          next
        j = int( buf )
        if( j > 0 )
        {
          c[j] = real( buf1 )
        }
      }

      fn_fit_tmp = fn_fit[k]

#      printf( "%12.9e %12.9e %12.9e %12.9e %12.9e %12.9e\n",\
#              c[1], c[2], c[3], c[4], c[5], c[6], >> fn_fit_tmp )

      printf( "%12.9e %12.9e %12.9e %12.9e\n",\
              c[1], c[2], c[3], c[4], >> fn_fit_tmp )

      delete( infn, ver- )
      delete( tmp_tbl, ver- )
    }
  }
  
end
