#
#       HBS REDUCTION SUB ROUTINE
#
#       Averaging flat-fielding images: hbsflatave.cl
#
#       1997/04/06      K.Kawabata   ver 1.00
#       1997/04/22      K.Kawabata   ver 1.10  pixel sensitivity
#       1997/09/03      K.Kawabata   ver 1.11  1hole diaphragm
#       1998/04/20      K.Kawabata   ver 2.00  unification
#
#  FLAT.xxx.itj.px  xxx = 1HL   1HOLE
#                       = 2HL   2HOLEs
#                       = 2S2   2SLIT02
#                     i = 1: first FLAT
#                     i = 2: second FLAT
#                     i = 3: second FLAT (UP-trail)
#                     j = theta / 22.5
#

procedure hbsflatave

begin

  int c, i, j, k
  file fn
  string fna[100], fnb
  string fnbhead = "AHO."
  string tmp_list, tmp_fn1, tmp_fn
  file flatlist
  int n_image
  int n_list
  string hbslist[50], hbsfile[50], diaph[50]
  int flattype[50], theta_n[50]
  string slit[50]
  string buf1, buf2, buf3, buf4, buf5, buf6
  string buf="y"
  string ftype1="absorption"
  string nullfile
  real fw1 = 25.
  real x1, x
  int flag_noreject = 1 # 0:apply sigma-rejection 1:no rejection
  
  print( "" )
  print( "--------------------------------" )
  print( " Averaging flat-fielding images" )
  print( "--------------------------------" )
  print( "" )

  nullfile = envget( "_nullfile" )
  flatlist = envget( "_flatlist" )
  tmp_list = mktemp( "hbs.tmp.list." )
  tmp_fn = mktemp( "hbs.tmp.image." )
  tmp_fn1 = mktemp( "hbs.tmp.image." )

#----- reject mode

  printf( "Select reject mode (0:apply sigma reject 1:no reject) >> " )
  k = scan( buf )
  while( buf != "0" && buf != "1" )
    k = scan( buf )
  if( buf == "0" )
    flag_noreject = 0
  else
    flag_noreject = 1

#----- Delete old flat list and flat images

  imdelete( fnbhead//"*.imh", ver- )

  if( access( flatlist ) == 0 )
  {
    printf( "Cannot find %s. Do hbsflatgroup.\n", flatlist )
    bye
  }

  sections( "FLAT.*.px.imh", option="fullname", > tmp_list )
  if( sections.nimages > 0 )
  {
    print( "There are old averaged flat images..." )
    type( tmp_list )
    printf( "delete them all ?  (y/n)  >> " )
    k = scan( buf )
    while( buf != "y" && buf != "n" )
       k = scan( buf )
    if( buf == "y" )
      imdelete( "@"//tmp_list )
    else
    {
      print( "Can't continue. Please delete or rename old images(FLAT.*.imh)" )
      bye
    }
  }
  delete( tmp_list ) 

#----- Scanning the FLAT list file (made by hbsflatgroup)

  n_list = 0
  list = flatlist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6) != EOF )
  {
    if( substr( buf2, 1, 4 ) != "FLAT" )
    {
      print( "The list-file "//flatlist//" is not produced by HBSGROUP.CL" )
      next
    }
    n_list += 1
    hbslist[n_list] = buf1
    hbsfile[n_list] = buf2
    flattype[n_list] = int( buf3 )
    theta_n[n_list] = int( buf4 )
    diaph[n_list] = buf5
    slit[n_list] = buf6
  }

#----- confirmation

  printf( "Detected these lists\n\n" )
  for( i=1; i<=n_list; i+=1 )
  {
    printf( "---- No.%d  %s ----\n", i, hbslist[i] )
    type( hbslist[i] )
    printf( "\n" )
  }

nextstep10:
  buf = "y"
  printf( "OK ?  [y]:yes [n]:no >> " )
  k = scan( buf )
  while( buf != "y" && buf != "n" )
    k = scan( buf )
  if( buf == "n" )
  {
    print( "Please correct the list files (FLAT.*) in an other terminal" )
    goto nextstep10
  }

#----- counting the images in a grouped list

  for( i=1; i<=n_list; i+=1 )
  {
    n_image = 0
    sed( 's/\.tr\./\.px\./', hbslist[i], > tmp_list )
    list = tmp_list
    while( fscan( list, fn ) != EOF )
    {
      n_image += 1
      fna[n_image] = fn
    }
    if( n_image == 0 )
      print( hbslist[i], " is empty. Something was wrong in grouping!!" )

#----- Averaging (or summing) and creating flat images

   if( flattype[i] == 4 )
   {
     print( "UP-trail images" )
     for( j=1; j<=n_image; j+=1 )
     {
       printf( "%2d : %s  ", j, fna[j] )
       fnb = fnbhead//j
       if( j == 1 )
       {
         imgets( fna[j], 'i_naxis2' )
         c = int( imgets.value )
	 printf( "%s %s %d\n", fna[j], tmp_fn1, c )
         blkavg( fna[j], tmp_fn1, b1=1, b2=c, option="sum" )
         imgets( fna[j], 'SEEING' )
         fw1 = real( imgets.value ) * 4.0 * 1.5

         If( access( "database/id"//tmp_fn1 ) )
           delete( "database/id"//tmp_fn1 )

         printf( "[m]:mark near the '%s' feature\n", ftype1 )

         identify( tmp_fn1, section="middle line", databas='database',\   
                   coordli=nullfile, nsum=1, match=20.,\
                   maxfeat=1, fwidth=fw1, cradius=40., functio='chebyshev',\
                   order=1, niterate=10, low_rej=3., high_rej=3., autowrite+,\
                   grow=0.,ftype=ftype1, mode='ql',thresho=1. )

         list = "database/id"//tmp_fn1
         c = fscan( list, buf, buf, buf, buf, buf, buf )
         c = fscan( list, buf, buf, buf, buf, buf, buf )
         while( fscan( list, buf1, buf, buf, buf, buf, buf ) != 6 ){}
         x1 = real( buf1 )
         printf( " %7.2f\n", x1 )
	 imcopy( fna[1], fnb, ver- )
       }
       else
       {
         imgets( fna[j], 'i_naxis2' )
         c = int( imgets.value )
         blkavg( fna[j], tmp_fn, b1=1, b2=c, option="sum" )

         if( access( "database/id"//tmp_fn ) )
           delete( "database/id"//tmp_fn )
	
         reidentify( tmp_fn1, tmp_fn, section="middle line",\
                     database="database", shift=0., step=1000, nsum=1,\
                     cradius=40., nlost=0, refit-, override+,\
                     coordli=nullfile, maxfeat=1,\
                     interactive-, answer+ ,threshold=threshold1 )
         while( !access( "database/id"//tmp_fn ) )
         {
           print( "reidentify failed in "//tmp_fn//".  please select" )
           print( "[r] retry identify   [m] manual input" )
           c = scan( buf )
           while( buf != "r" && buf !="m" )
             c = scan( buf )
           if( buf == "r" )
           {
             printf( "mark with 'm' on the %s feature\n", ftype1 )

             identify( tmp_fn, section="middle line", databas='database',\   
                       coordli=nullfile, nsum=1, match=20., maxfeat=1,\
                       fwidth=fw1, cradius=40., functio='chebyshev',\
                       order=1, niterate=10, low_rej=3., high_rej=3.,\
                       autowrite+,\
                       grow=0.,ftype=ftype1, mode='ql',thresho=1. )
           }
           else
           {
             printf( "read y-coordinate on the first feature (%s)\n", ftype1 )

             identify( tmp_fn, section="middle line", databas='database',\   
                       coordli=nullfile, nsum=1, match=20., maxfeat=1,\
                       fwidth=fw1, cradius=40., functio='chebyshev',\
                       order=1, niterate=10, low_rej=3., high_rej=3.,\
	               autowrite+,\
                       grow=0.,ftype=ftype1, mode='ql',thresho=1. )

             printf( "input x-coodinate >> " )
             c = scan( x )
             goto step2
           }
         }

         list = "database/id"//tmp_fn
         c = fscan( list, buf, buf, buf, buf, buf, buf )
         c = fscan( list, buf, buf, buf, buf, buf, buf )
         while( fscan( list, buf1, buf, buf, buf, buf, buf ) != 6 ){}
         x = real( buf1 )

step2 :
         printf( " %7.2f   %5.2f\n", x, (x-x1) )
         imshift( fna[j], fnb, (x1-x), 0., interp_type="linear",\
                  boundary="nearest", constant=0. )
	 imdelete( tmp_fn, ver- )
	}
      }
       
      imdelete( tmp_fn1, ver- )
      print( "Now summing images in "//hbslist[i]//" (", n_image, "images)" )
      imsum( input=fnbhead//"*.imh", output=hbsfile[i], hparams="EXPTIME",\
             pixtype="real", calctype="real", option="sum",\
             low_reject=0., high_reject=0., ver- )
      imdelete( fnbhead//"*.imh", ver- )
      imreplace( hbsfile[i], 0.0, upper=0.0 )
    }
    else
    {
#     if( flattype[i] == 3 )
#     {
#       print( "2nd-flat images" )
#       for( j=1; j<=n_image; j+=1 )
#       {
#         printf( "%2d : %s  ", j, fna[j] )
#         fnb = fnbhead//j
#         if( j == 1 )
#         {
#           imgets( fna[j], 'i_naxis2' )
#           c = int( imgets.value )
#           printf( "%s %s %d\n", fna[j], tmp_fn1, c )
#           blkavg( fna[j], tmp_fn1, b1=1, b2=c, option="sum" )
#           imgets( fna[j], 'SEEING' )
#           fw1 = real( imgets.value ) * 4.0 * 1.5

#           If( access( "database/id"//tmp_fn1 ) )
#             delete( "database/id"//tmp_fn1 )

#      }
#      else
#      {
        print( "Now averaging images in "//hbslist[i]//" (", n_image, "images)" )

        if( flag_noreject == 0 )
        {
          if( n_image < 3 )
          {
            imcombine( input="@"//tmp_list, output=hbsfile[i],\
	               combine="average",\
                       reject="none", project-, outtype="real",\
                       offsets="none", masktype="none", scale="none",\
                       zero="none", weight="none", lthreshold=INDEF,\
                       hthreshold=INDEF, nlow=1, nhigh=1 )
          }
          else
          {
            imcombine( input="@"//tmp_list, output=hbsfile[i],\
                       combine="average",\
                       reject="sigclip", project-, outtype="real",\
                       offsets="none", masktype="none", scale="none",\
                       zero="none", weight="none", lthreshold=INDEF,\
                       hthreshold=INDEF, nlow=1, nhigh=1, nkeep=1,\
                       mclip+, lsigma=3.0, hsigma=3.0, sigscale=0.1 )
          }
        } 
        else
        {
          imcombine( input="@"//tmp_list, output=hbsfile[i],\
                     combine="average",\
                     reject="none", project-, outtype="real",\
                     offsets="none", masktype="none", scale="none",\
                     zero="none", weight="none", lthreshold=INDEF,\
                     hthreshold=INDEF, nlow=1, nhigh=1 )
        }
	imreplace( hbsfile[i], 0.0, upper=0.0 )
#      }
    }
    delete( tmp_list, ver- )
  }

end
