#
#       HBS REDUCTION SUB ROUTINE
#
#       make i, q and u spectrum using UP spectra : hbsiqumake.cl
#
#       1997/05/02      K.Kawabata  Ver.0.10  for CRYOCAM manual reduc
#       1997/09/08      K.Kawabata  Ver.1.00  use 'HBS.list'
#       1998/04/20      K.Kawabata  Ver.2.00  unification
#       1998/05/31      K.Kawabata  Ver.2.10
#       2000/02/29      K.Kawabata  Ver.3.00  for SITe-CCD
#       2000/03/25      K.Kawabata  Ver.3.01  weighted mean
#

procedure hbsiqumake

begin

  file iqulist
  string hbslist[250]
  string khead[250]
  string klist0, klist1, klist2, klist3
  string qlist[250], ulist[250], ilist[250], diaph[250], filter[250]
  int max_theta_n[250]
  int c, i, j, n_list
  int x1, x2
  file logfn

  file fn1e, fn1o, fn2e, fn2o, qu_fn, i_fn1, i_fn2, kap_fn
  file weightlist
  string odall
  real geadu
  real pregain, postgain, geff
  real mflux
  file fn1o_tmp
  string qumode
  real flag_01    # 1: 0.0d or 45.0d image  -1: 22.5d or 67.5d image
  file fullkappa, kappa, kappa_bin, binfn
  file fullkappas, kappas # in case of no '.red' file
  file fullsens, sens, sensti5, senssite5
  string kappaext # kappa image name extension
  string kapbuf, spmode
  file prmfile, binmode

  file tmp_list, tmp_fn1, tmp_fn2, tmp_fn3
  string buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7, buf8
  string caldir, etcdir, fnphot
  file fn_def

  int flag_uplow  #  0:lower  1: upper  -1:other
  int flag_iqu
  int flag_calc   #  0:normal 1:e/o ratio
  int n_version      # IRAF version (210 or 211)

#--- initial settings

  logfn = envget( "_logfile" )
  caldir = envget( "_caldir" )
  fn_def = caldir//envget( "_defhname" )

  if( access( "hbs.tmp.time" ) == 1 )
    delete( "hbs.tmp.time", ver- )
  time >> hbs.tmp.time
  list = "hbs.tmp.time"
  if( fscan( list, buf, buf1, buf2 ) != 3 )
  {
    printf( "Error: time task -- reading\n" )
    bye
  }
  delete( "hbs.tmp.time", ver- )

  print( "" )
  printf( "%s %s\n", buf1, buf2 )
  print( "----------------------" )
  print( " make i, q, u spectra" )
  print( "----------------------" )
  print( "" )

  print( "", >> logfn )
  printf( "%s %s\n", buf1, buf2, >> logfn )
  print( "----------------------", >> logfn )
  print( " make i, q, u spectra", >> logfn )
  print( "----------------------", >> logfn )
  print( "", >> logfn )

  iqulist = envget( "_iqulist" )
  caldir = envget( "_caldir" )
  etcdir = envget( "_etcdir" )
  flag_iqu = int( envget( "_iqucalc" ) )
  sensti5 = envget( "_sensTI5" )
  senssite5 = envget( "_sensSITe5" )
  fnphot = caldir//envget( "_photmet" )

  tmp_list = mktemp( "hbs.tmp.list." )
  tmp_fn1 = mktemp( "hbs.tmp.image." )
  tmp_fn2 = mktemp( "hbs.tmp.image." )
  tmp_fn3 = mktemp( "hbs.tmp.image." )

#----- read difinition file

  buf1 = ""
  buf2 = ""
  buf3 = ""
  x1 = 0
  x2 = 0

  if( access( fn_def ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_def )
    printf( "Cannot find '%s'\n", fn_def, >> logfn )
    bye
  }

  list = fn_def
  while( fscan( list, buf1, buf2, buf3 ) != EOF )
  {
    if( buf2 == "NX3" )
    {
      x1 = int( buf3 ) * 0.4
      x2 = int( buf3 ) * 0.6
    }
    if( buf2 == "GEADU" )
      geadu = real( buf3 )
  }
  if( x1 == 0 || x2 == 0 )
  {
    printf( "Read error in '%s'\n", fn_def )
    printf( "Read error in '%s'\n", fn_def, >> logfn )
    bye
  }

#----- IRAF version check

  buf1 = envget( "version" )
  c = stridx( "V", buf1 )
  buf2 = substr( buf1, c, ( c + 4 ) )

  n_version = 0

  if( buf2 == "V2.11" )
  {
    n_version = 211
  }
  else
  {
    if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'hbsred'\n" )
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n" )
    printf( "IRAF version check failed.\n", >> logfn )
    printf( "Please input like next line, and restart 'hbsred'\n", >> logfn )
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n", >> logfn )
    bye
  }

#----- Scanning the IQU-list file

  if( access( iqulist ) == 0 )
  {
    print( "Cannot find "//iqulist//"   Please do hbsiqugroup" )
    print( "Cannot find "//iqulist//"   Please do hbsiqugroup", >> logfn )
    bye
  }

  n_list = 0
  list = iqulist

  while( fscan( list, buf1, buf2, buf3, buf4, buf5, buf6, buf7, buf8 ) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "HBS" )
    {
      print( "The list-file ", iqulist, " is not produced by HBSiquGROUP.CL" )
      print( "The list-file ", iqulist, " is not produced by HBSiquGROUP.CL",\
             >> logfn )
      bye
    }
    n_list += 1
    hbslist[n_list] = buf1
    ilist[n_list] = buf2
    qlist[n_list] = buf3
    ulist[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    max_theta_n[n_list] = int( buf7 )
    khead[n_list] = buf8
  }

#----- main routine ----

  for( i=1; i<=n_list; i+=1 )
  {

  #----- diaphragm

#    if( substr( diaph[i], 1, 3 ) == "2HL" )
#    {
#    }
#    if( substr( diaph[i], 1, 3 ) == "2S2" )
#    {
#    }

    c = strlen( diaph[i] )
    if( substr( diaph[i], 4, 4 ) == "L" )
    {
      flag_uplow = 0
      printf( "--- OBJECT LIST : %s  LOWER  ---\n",\
	      hbslist[i] )
      printf( "--- OBJECT LIST : %s  LOWER  ---\n",\
	      hbslist[i], >> logfn )
    }
    else
    {
      if( substr( diaph[i], 4, 4 ) == "U" )
      {
        flag_uplow = 1
        printf( "--- OBJECT LIST : %s  UPPER  ---\n",\
	        hbslist[i] )
        printf( "--- OBJECT LIST : %s  UPPER  ---\n",\
	        hbslist[i], >> logfn )
      }
      else
      {
        flag_uplow = -1
        printf( "%s did not used two diaphragm mode\n", hbslist[i] )
        printf( "%s did not used two diaphragm mode\n", hbslist[i], >> logfn )
	next
      }
    }
    prmfile = hbslist[i]//".prm"

    list = prmfile
    if( fscan( list, buf, buf, buf, binmode, spmode, buf, buf ) != 7 )
    {
      printf( "Read error in '%s'\n", prmfile )
      printf( "Read error in '%s'\n", prmfile, >> logfn )
      bye
    }
    kapbuf = substr( spmode, 1, 1 )

#----- Kappa mode

#      buf = ""
#      printf( "Select Sp type of %s [1]:OBAFGK [2]:M >> ", hbslist[i] )
#      c = scan( buf )
#      while( buf != "1" && buf != "2" )
#        c = scan( buf )
      if( kapbuf == "1" )
        kappaext = ""
      else
        kappaext = ".red"

    binfn = hbslist[i]//".binew"
   

#----- Self-calibration mode

    if( max_theta_n[i] == 4 )
    {

#----- read indivisual list file

      if( access( qlist[i] ) == 0 )
      {
        printf( "Not found '%s'\n", qlist[i] )
        printf( "Not found '%s'\n", qlist[i], >> logfn )
        next
      }
      if( access( ulist[i] ) == 0 )
      {
        printf( "Not found '%s'\n", ulist[i] )
        printf( "Not found '%s'\n", ulist[i], >> logfn )
        next
      }
      if( access( ilist[i] ) == 0 )
      {
        printf( "Not found '%s'\n", ilist[i] )
        printf( "Not found '%s'\n", ilist[i], >> logfn )
        next
      }
      klist0 = khead[i]//"0"
      klist1 = khead[i]//"1"
      klist2 = khead[i]//"2"
      klist3 = khead[i]//"3"
      if( access( klist0 ) == 0 )
      {
        printf( "Not found '%s'\n", klist0 )
        printf( "Not found '%s'\n", klist0, >> logfn )
        next
      }
      if( access( klist1 ) == 0 )
      {
        printf( "Not found '%s'\n", klist1 )
        printf( "Not found '%s'\n", klist1, >> logfn )
        next
      }
      if( access( klist2 ) == 0 )
      {
        printf( "Not found '%s'\n", klist2 )
        printf( "Not found '%s'\n", klist2, >> logfn )
        next
      }
      if( access( klist3 ) == 0 )
      {
        printf( "Not found '%s'\n", klist3 )
        printf( "Not found '%s'\n", klist3, >> logfn )
        next
      }

#----- Calculation Mode

      flag_calc = 0

      buf1 = substr( diaph[i], 1, 3 )
      if( buf1 == "2S2" )
      {
        flag_calc = 1
      }
      else
      {
        if( flag_iqu == 1 )
	  flag_calc = 1
      }

  #----- calculation of I (unbinned)

      printf( "---- make i-images (unbinned and binned) ----\n" )
      printf( "---- make i-images (unbinned and binned) ----\n", >> logfn )

      list = ilist[i]
      while( fscan( list, buf1, buf2, i_fn1, i_fn2 ) != EOF )
      {
	kappa = buf2//kappaext
	fullkappa = caldir//kappa

        printf( " %s %s (%s)\n", i_fn1, i_fn2, kappa )
        printf( " %s %s (%s)\n", i_fn1, i_fn2, kappa, >> logfn )

        fn1e = buf1//"co.e"
        fn1o = buf1//"co.o"

        if( access( fullkappa//".imh" ) == 0 )
	{
	  buf8 = substr( fullkappa, 1, strlen( fullkappa ) - 4 )
	  if( access( buf8//".imh" ) == 1 )
	  {
	    printf( "Substituting %s for %s.\n", buf8, kappa )
	    printf( "Substituting %s for %s.\n", buf8, kappa, >> logfn )
	    fullkappa = buf8
	  }
	  else
	  {
	    printf( "Cannot find %s. Please check calibration files.\n",\
	    fullkappa )
	    printf( "Cannot find %s. Please check calibration files.\n",\
	    fullkappa, >> logfn )
	    return
	  }
	}

	imgets( fn1e, "DETECTOR" )
        buf3 = imgets.value
	if( buf3 == "1024x1024 TI CCD" || buf3 == "T5 CRYOCAM (TC215)" )
	{
	  sens = sensti5
	}
	else
	{
	  if( buf3 == "S5 CRYOCAM (SITe)" )
	  {
	    sens = senssite5
	  }
	  else
	  {
	    printf( "Header keyword 'DETECTOR' wrong (%s)\n", imgets.value )
	    printf( "TI CCD is assumed.\n" )
	    printf( "Header keyword 'DETECTOR' wrong (%s)\n", imgets.value,\
                    >> logfn )
	    printf( "TI CCD is assumed.\n", >> logfn )
	    sens = sensti5
	  }
	}

	imgets( fn1e, "PREGAIN" )
        pregain = real( imgets.value )
	imgets( fn1e, "POSTGAIN" )
        postgain = real( imgets.value )
	geff = geadu * pregain * postgain

	fullsens = caldir//sens
#        printf( " ( %s + %s * %s ) / %s --->  %s\n", fn1e, fn1o, kappa,\
#	        sens, i_fn2 )

	if( access( fn1e//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn1e )
	  printf( "Not found '%s'. Skip.\n", fn1e, >> logfn )
	  next
	}
	if( access( fn1o//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn1o )
	  printf( "Not found '%s'. Skip.\n", fn1o, >> logfn )
	  next
	}

	if( access( i_fn2//".imh" ) == 1 )
	{
	  imdelete( i_fn2, ver- )
	}

        imarith( fn1o, "*", fullkappa, tmp_fn1,\
                 pixtype="real", calctype="real", ver- )
        imarith( fn1e, "+", tmp_fn1, i_fn2,\
                 pixtype="real", calctype="real", ver- )

        if( access( fullsens//".imh" ) == 1 )
	{
          imarith( i_fn2, "/", fullsens, i_fn2,\
                   pixtype="real", calctype="real", ver- )
	}
	else
	{
	  printf( "Sens func: '%s' not found. Assumed to 1\n", fullsens )
	  printf( "Sens func: '%s' not found. Assumed to 1\n",\
                  fullsens, >> logfn )
        }
        imdelete( tmp_fn1, ver- )

#---- calculation of i-image (binned)

        if( access( i_fn2 ) == 1 )
	  delete( i_fn2, ver- )
        if( access( i_fn1 ) == 1 )
	  delete( i_fn1, ver- )

        if( access( i_fn1//".imh" ) == 1 )
	  imdelete( i_fn1, ver- )

        if( n_version == 210 )
        { 
          wfits( i_fn2//".imh", i_fn2, bscale=1.0, bzero=0.0, long_header-,
                 short_header-, bitpix=-32, scale-, autoscale- )
        }
        else
	{
          if( n_version == 211 )
          {
            wfits( i_fn2//".imh", i_fn2, bscale=1.0, bzero=0.0,\
	           long_header-,
                   fextn="", short_header-, bitpix=-32, scale-, autoscale- )
          }
          else
          {
            printf( "IRAF version error (%d)\n", n_version )
            printf( "IRAF version error (%d)\n", n_version, >> logfn )
            bye
          }
	}
	copy( i_fn2, i_fn1, ver- )
	hbscbin( i_fn2, binfn, etcdir, fnphot, i_fn1, binmode )
	delete( i_fn2, ver- )

        if( n_version == 210 )
        {
          rfits( i_fn1, "", i_fn1, make_image+, long_header-,\
	         short_header-,\
                 datatype="real", blank=0., scale-, oldirafname-, offset=0 )
        }
        if( n_version == 211 )
        {
          rfits( i_fn1, "", i_fn1//".imh", make_image+, long_header-,\
	         short_header-,\
                 datatype="real", blank=0., scale-, oldirafname-, offset=0 )
        }
	delete( i_fn1, ver- )
        hbscoord( i_fn1 )

      }

#---- q-image

      printf( "---- make q-images ----\n" )
      printf( "---- make q-images ----\n", >> logfn )
      list = qlist[i]
      weightlist = qlist[i]//"w"
      if( access( weightlist ) == 1 )
	delete( weightlist, ver- )
      while( fscan( list, qumode, buf1, buf2, qu_fn ) != EOF )
      {
        if( qumode != "SELF" )
	{
	  printf( "Error: HBSIQUMAKE failure in '%s'. Check Images", qlist[i] )
	  printf( "Error: HBSIQUMAKE failure in '%s'. Check Images", qlist[i],\
                  >> logfn )
	  break
	}

        printf( " %s %s  --->  %s\n", buf1, buf2, qu_fn )
        printf( " %s %s  --->  %s\n", buf1, buf2, qu_fn, >> logfn )
        fn1e = buf1//"bn.e"
        fn1o = buf1//"bn.o"
        fn2e = buf2//"bn.e"
        fn2o = buf2//"bn.o"
	odall = buf1//"co.e,"//buf1//"co.o,"//buf2//"co.e,"//buf2//"co.o"

	if( access( fn1e//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn1e )
	  printf( "Not found '%s'. Skip.\n", fn1e, >> logfn )
	  next
	}
	if( access( fn1o//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn1o )
	  printf( "Not found '%s'. Skip.\n", fn1o, >> logfn )
	  next
	}
	if( access( fn2e//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn2e )
	  printf( "Not found '%s'. Skip.\n", fn2e, >> logfn )
	  next
	}
	if( access( fn2o//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn2o )
	  printf( "Not found '%s'. Skip.\n", fn2o, >> logfn )
	  next
	}

	if( access( qu_fn//".imh" ) == 1 )
	{
	  imdelete( qu_fn, ver- )
	}

  #----- calculation of Stokes parameters: Q/I

        # make weight list
        imsum( odall, tmp_fn1, pixtype="real", calctype="real",\
	       option="sum", ver- )
        imstat( tmp_fn1//"["//x1//":"//x2//"]", fields="midpt",\
	        format- ) | scan( mflux )
        imdelete( tmp_fn1, ver- )
	printf( "%f\n", ( mflux / ( 1000000.0 * geff ) ), >> weightlist )
  
        if( flag_calc == 1 )
	{
          imarith( fn1e, "/", fn1o, tmp_fn1,\
                   pixtype="real", calctype="real", ver- )
          imarith( fn2o, "/", fn2e, tmp_fn2,\
                   pixtype="real", calctype="real", ver- )
          imarith( tmp_fn1, "*", tmp_fn2, tmp_fn1,\
                   pixtype="real", calctype="real", ver- )
          imdelete( tmp_fn2, ver- )
	  imfunction( tmp_fn1, tmp_fn1, "sqrt", ver- )
          imarith( 1.0, "-", tmp_fn1, qu_fn,\
                   pixtype="real", calctype="real", ver- )
          imarith( 1.0, "+", tmp_fn1, tmp_fn2,\
                   pixtype="real", calctype="real", ver- )
          imarith( qu_fn, "/", tmp_fn2, qu_fn,\
                   pixtype="real", calctype="real", ver- )
          imdelete( tmp_fn1, ver- )
          imdelete( tmp_fn2, ver- )
	}
	else
	{
          imarith( fn1e, "-", fn2e, tmp_fn1,\
                   pixtype="real", calctype="real", ver- )
          imarith( fn1e, "+", fn2e, tmp_fn2,\
                   pixtype="real", calctype="real", ver- )
          imarith( tmp_fn1, "/", tmp_fn2, tmp_fn1,\
                   pixtype="real", calctype="real", ver- )
          imdelete( tmp_fn2, ver- )
          imarith( fn1o, "-", fn2o, tmp_fn2,\
                   pixtype="real", calctype="real", ver- )
          imarith( fn1o, "+", fn2o, tmp_fn3,\
                   pixtype="real", calctype="real", ver- )
          imarith( tmp_fn2, "/", tmp_fn3, tmp_fn2,\
                   pixtype="real", calctype="real", ver- )
          imdelete( tmp_fn3, ver- )
          imsum( tmp_fn1//","//tmp_fn2, qu_fn,\
                   pixtype="real", calctype="real", ver- )
          imarith( qu_fn, "/", 2.0, qu_fn,\
                   pixtype="real", calctype="real", ver- )
          printf( "%s qu-1: ", qu_fn )
          printf( "%s qu-1: ", qu_fn, >> logfn )
          imstat( tmp_fn1//"["//x1//":"//x2//"]", fields="mean,stddev", format- )
          imstat( tmp_fn1//"["//x1//":"//x2//"]", fields="mean,stddev", format-,\
                  >> logfn )
          printf( "%s qu-2: ", qu_fn )
          printf( "%s qu-2: ", qu_fn, >> logfn )
          imstat( tmp_fn2//"["//x1//":"//x2//"]", fields="mean,stddev", format- )
          imstat( tmp_fn2//"["//x1//":"//x2//"]", fields="mean,stddev", format-,\
                  >> logfn )
          imdelete( tmp_fn1, ver- )
          imdelete( tmp_fn2, ver- )
          imarith( qu_fn, "/", 2.0, qu_fn,\
                   pixtype="real", calctype="real", ver- )
	}
      }

#---- u-image

      printf( "---- make u-images ----\n" )
      printf( "---- make u-images ----\n", >> logfn )
      list = ulist[i]
      weightlist = ulist[i]//"w"
      if( access( weightlist ) == 1 )
	delete( weightlist, ver- )
      while( fscan( list, qumode, buf1, buf2, qu_fn ) != EOF )
      {
        if( qumode != "SELF" )
	{
	  printf( "Error: HBSIQUMAKE failure in '%s'. Check Images", ulist[i] )
	  printf( "Error: HBSIQUMAKE failure in '%s'. Check Images", ulist[i],\
                  >> logfn )
	  break
	}

        printf( " %s %s  --->  %s\n", buf1, buf2, qu_fn )
        printf( " %s %s  --->  %s\n", buf1, buf2, qu_fn, >> logfn )
        fn1e = buf1//"bn.e"
        fn1o = buf1//"bn.o"
        fn2e = buf2//"bn.e"
        fn2o = buf2//"bn.o"
	odall = buf1//"co.e,"//buf1//"co.o,"//buf2//"co.e,"//buf2//"co.o"

	if( access( fn1e//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn1e )
	  printf( "Not found '%s'. Skip.\n", fn1e, >> logfn )
	  next
	}
	if( access( fn1o//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn1o )
	  printf( "Not found '%s'. Skip.\n", fn1o, >> logfn )
	  next
	}
	if( access( fn2e//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn2e )
	  printf( "Not found '%s'. Skip.\n", fn2e, >> logfn )
	  next
	}
	if( access( fn2o//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn2o )
	  printf( "Not found '%s'. Skip.\n", fn2o, >> logfn )
	  next
	}

	if( access( qu_fn//".imh" ) == 1 )
	{
	  imdelete( qu_fn, ver- )
	}

  #----- calculation of Stokes parameters

        # make weight list
        imsum( odall, tmp_fn1, pixtype="real", calctype="real",\
	       option="sum", ver- )
        imstat( tmp_fn1//"["//x1//":"//x2//"]", fields="midpt",\
	        format- ) | scan( mflux )
        imdelete( tmp_fn1, ver- )
	printf( "%f\n", ( mflux / ( 1000000.0 * geff ) ), >> weightlist )
  
        if( flag_calc == 1 )
	{
          imarith( fn1e, "/", fn1o, tmp_fn1,\
                   pixtype="real", calctype="real", ver- )
          imarith( fn2o, "/", fn2e, tmp_fn2,\
                   pixtype="real", calctype="real", ver- )
          imarith( tmp_fn1, "*", tmp_fn2, tmp_fn1,\
                   pixtype="real", calctype="real", ver- )
          imdelete( tmp_fn2, ver- )
	  imfunction( tmp_fn1, tmp_fn1, "sqrt", ver- )
          imarith( 1.0, "-", tmp_fn1, qu_fn,\
                   pixtype="real", calctype="real", ver- )
          imarith( 1.0, "+", tmp_fn1, tmp_fn2,\
                   pixtype="real", calctype="real", ver- )
          imarith( qu_fn, "/", tmp_fn2, qu_fn,\
                   pixtype="real", calctype="real", ver- )
          imdelete( tmp_fn1, ver- )
          imdelete( tmp_fn2, ver- )
	}
	else
	{
          imarith( fn1e, "-", fn2e, tmp_fn1,\
                   pixtype="real", calctype="real", ver- )
          imarith( fn1e, "+", fn2e, tmp_fn2,\
                   pixtype="real", calctype="real", ver- )
          imarith( tmp_fn1, "/", tmp_fn2, tmp_fn1,\
                   pixtype="real", calctype="real", ver- )
          imdelete( tmp_fn2, ver- )
          imarith( fn1o, "-", fn2o, tmp_fn2,\
                   pixtype="real", calctype="real", ver- )
          imarith( fn1o, "+", fn2o, tmp_fn3,\
                   pixtype="real", calctype="real", ver- )
          imarith( tmp_fn2, "/", tmp_fn3, tmp_fn2,\
                   pixtype="real", calctype="real", ver- )
          imdelete( tmp_fn3, ver- )
          imarith( tmp_fn1, "+", tmp_fn2, qu_fn,\
                   pixtype="real", calctype="real", ver- )
          printf( "%s qu-1: ", qu_fn )
          printf( "%s qu-1: ", qu_fn, >> logfn )
          imstat( tmp_fn1//"["//x1//":"//x2//"]", fields="mean,stddev", format- )
          imstat( tmp_fn1//"["//x1//":"//x2//"]", fields="mean,stddev", format-,\
                  >> logfn )
          printf( "%s qu-2: ", qu_fn )
          printf( "%s qu-2: ", qu_fn, >> logfn )
          imstat( tmp_fn2//"["//x1//":"//x2//"]", fields="mean,stddev", format- )
          imstat( tmp_fn2//"["//x1//":"//x2//"]", fields="mean,stddev", format-,\
                  >> logfn )
          imdelete( tmp_fn1, ver- )
          imdelete( tmp_fn2, ver- )
          imarith( qu_fn, "/", 2.0, qu_fn,\
                   pixtype="real", calctype="real", ver- )
	}
      }

#---- kappa-image

      printf( "---- make kappa-images ----\n" )
      printf( "---- make kappa-images ----\n", >> logfn )
      list = klist0
      weightlist = klist0//"w"
      if( access( weightlist ) == 1 )
	delete( weightlist, ver- )
      while( fscan( list, buf1, kap_fn ) != EOF )
      {
        fn1e = buf1//"bn.e"
        fn1o = buf1//"bn.o"
        printf( " %s %s  --->  %s\n", fn1e, fn1o, kap_fn )
        printf( " %s %s  --->  %s\n", fn1e, fn1o, kap_fn, >> logfn )
	odall = buf1//"co.e,"//buf1//"co.o"

	if( access( fn1e//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn1e )
	  printf( "Not found '%s'. Skip.\n", fn1e, >> logfn )
	  next
	}
	if( access( fn1o//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn1o )
	  printf( "Not found '%s'. Skip.\n", fn1o, >> logfn )
	  next
	}

	if( access( kap_fn//".imh" ) == 1 )
	{
	  imdelete( kap_fn, ver- )
	}
        # make weight list
        imsum( odall, tmp_fn1,\
                 pixtype="real", calctype="real", option="sum", ver- )
        imstat( tmp_fn1//"["//x1//":"//x2//"]", fields="midpt",\
	        format- ) | scan( mflux )
        imdelete( tmp_fn1, ver- )
	printf( "%f\n", ( mflux / ( 1000000.0 * geff ) ), >> weightlist )
  
        imarith( fn1e, "/", fn1o, kap_fn, divzero = 1.,\
                 pixtype="real", calctype="real", ver- )
      }

      list = klist1
      weightlist = klist1//"w"
      if( access( weightlist ) == 1 )
	delete( weightlist, ver- )
      while( fscan( list, buf1, kap_fn ) != EOF )
      {
        fn1e = buf1//"bn.e"
        fn1o = buf1//"bn.o"
        printf( " %s %s  --->  %s\n", fn1e, fn1o, kap_fn )
        printf( " %s %s  --->  %s\n", fn1e, fn1o, kap_fn, >> logfn )
	odall = buf1//"co.e,"//buf1//"co.o"

	if( access( fn1e//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn1e )
	  printf( "Not found '%s'. Skip.\n", fn1e, >> logfn )
	  next
	}
	if( access( fn1o//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn1o )
	  printf( "Not found '%s'. Skip.\n", fn1o, >> logfn )
	  next
	}

	if( access( kap_fn//".imh" ) == 1 )
	{
	  imdelete( kap_fn, ver- )
	}
        # make weight list
        imsum( odall, tmp_fn1,\
                 pixtype="real", calctype="real", option="sum", ver- )
        imstat( tmp_fn1//"["//x1//":"//x2//"]", fields="midpt",\
	        format- ) | scan( mflux )
        imdelete( tmp_fn1, ver- )
	printf( "%f\n", ( mflux / ( 1000000.0 * geff ) ), >> weightlist )
  
        imarith( fn1e, "/", fn1o, kap_fn, divzero = 1.,\
                 pixtype="real", calctype="real", ver- )
      }

      list = klist2
      weightlist = klist2//"w"
      if( access( weightlist ) == 1 )
	delete( weightlist, ver- )
      while( fscan( list, buf1, kap_fn ) != EOF )
      {
        fn1e = buf1//"bn.e"
        fn1o = buf1//"bn.o"
        printf( " %s %s  --->  %s\n", fn1e, fn1o, kap_fn )
        printf( " %s %s  --->  %s\n", fn1e, fn1o, kap_fn, >> logfn )
	odall = buf1//"co.e,"//buf1//"co.o"

	if( access( fn1e//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn1e )
	  next
	}
	if( access( fn1o//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn1o )
	  printf( "Not found '%s'. Skip.\n", fn1o, >> logfn )
	  next
	}

	if( access( kap_fn//".imh" ) == 1 )
	{
	  imdelete( kap_fn, ver- )
	}
        # make weight list
        imsum( odall, tmp_fn1,\
                 pixtype="real", calctype="real", option="sum", ver- )
        imstat( tmp_fn1//"["//x1//":"//x2//"]", fields="midpt",\
	        format- ) | scan( mflux )
        imdelete( tmp_fn1, ver- )
	printf( "%f\n", ( mflux / ( 1000000.0 * geff ) ), >> weightlist )
  
        imarith( fn1e, "/", fn1o, kap_fn, divzero = 1.,\
                 pixtype="real", calctype="real", ver- )
      }

      list = klist3
      weightlist = klist3//"w"
      if( access( weightlist ) == 1 )
	delete( weightlist, ver- )
      while( fscan( list, buf1, kap_fn ) != EOF )
      {
        fn1e = buf1//"bn.e"
        fn1o = buf1//"bn.o"
        printf( " %s %s  --->  %s\n", fn1e, fn1o, kap_fn )
        printf( " %s %s  --->  %s\n", fn1e, fn1o, kap_fn, >> logfn )
	odall = buf1//"co.e,"//buf1//"co.o"

	if( access( fn1e//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn1e )
	  printf( "Not found '%s'. Skip.\n", fn1e, >> logfn )
	  next
	}
	if( access( fn1o//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn1o )
	  printf( "Not found '%s'. Skip.\n", fn1o, >> logfn )
	  next
	}

	if( access( kap_fn//".imh" ) == 1 )
	{
	  imdelete( kap_fn, ver- )
	}
        # make weight list
        imsum( odall, tmp_fn1,\
                 pixtype="real", calctype="real", option="sum", ver- )
        imstat( tmp_fn1//"["//x1//":"//x2//"]", fields="midpt",\
	        format- ) | scan( mflux )
        imdelete( tmp_fn1, ver- )
	printf( "%f\n", ( mflux / ( 1000000.0 * geff ) ), >> weightlist )
  
        imarith( fn1e, "/", fn1o, kap_fn, divzero = 1.,\
                 pixtype="real", calctype="real", ver- )
      }

    }


#---- In case of only two position angles to be used
    else
    {
#      printf( "Only two angle of half-wave plate. Not supported ye\n" )
#      printf( "\n" )
#      next


#----- read indivisual list file

      if( access( ilist[i] ) == 0 )
      {
        printf( "Not found '%s'\n", ilist[i] )
        printf( "Not found '%s'\n", ilist[i], >> logfn )
        next
      }
      klist0 = khead[i]//"0"
      klist1 = khead[i]//"1"
      klist2 = khead[i]//"2"
      klist3 = khead[i]//"3"
#      if( access( klist0 ) == 0 )
#      {
#        printf( "Not found '%s'\n", klist0 )
#        next
#      }

#----- Calculation Mode

      flag_calc = 1


  #----- calculation of I (unbinned)

      printf( "---- make i-images (unbinned and binned) ----\n" )
      printf( "---- make i-images (unbinned and binned) ----\n", >> logfn )

      list = ilist[i]
      while( fscan( list, buf1, buf2, i_fn1, i_fn2 ) != EOF )
      {
	kappa = buf2//kappaext
	fullkappa = caldir//kappa

        printf( " %s %s (%s)\n", i_fn1, i_fn2, kappa )
        printf( " %s %s (%s)\n", i_fn1, i_fn2, kappa, >> logfn )

        fn1e = buf1//"co.e"
        fn1o = buf1//"co.o"

        if( access( fullkappa//".imh" ) == 0 )
	{
	  buf8 = substr( fullkappa, 1, strlen( fullkappa ) - 4 )
	  if( access( buf8//".imh" ) == 1 )
	  {
	    printf( "Substituting %s for %s.\n", buf8, kappa )
	    printf( "Substituting %s for %s.\n", buf8, kappa, >> logfn )
	    fullkappa = buf8
	  }
	  else
	  {
	    printf( "Cannot find %s. Please check calibration files.\n",\
	    fullkappa )
	    printf( "Cannot find %s. Please check calibration files.\n",\
	    fullkappa, >> logfn )
	    return
	  }
	}


	imgets( fn1e, "DETECTOR" )
	if( imgets.value == "1024x1024 TI CCD" )
	{
	  sens = sensti5
	}
	else
	{
	  if( imgets.value == "S5 CRYOCAM (SITe)" )
	  {
	    sens = senssite5
	  }
	  else
	  {
	    printf( "Header keyword 'DETECTOR' wrong (%s)\n", imgets.value )
	    printf( "TI CCD is assumed.\n" )
	    printf( "Header keyword 'DETECTOR' wrong (%s)\n", imgets.value,\
                    >> logfn )
	    printf( "TI CCD is assumed.\n", >> logfn )
	    sens = sensti5
	  }
	}

        imgets( fn1e, "PREGAIN" )
        pregain = real( imgets.value )
        imgets( fn1e, "POSTGAIN" )
        postgain = real( imgets.value )
        geff = geadu * pregain * postgain


	fullsens = caldir//sens
#        printf( " ( %s + %s * %s ) / %s --->  %s\n", fn1e, fn1o, kappa,\
#	        sens, i_fn2 )

	if( access( fn1e//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn1e )
	  printf( "Not found '%s'. Skip.\n", fn1e, >> logfn )
	  next
	}
	if( access( fn1o//".imh" ) == 0 )
	{
	  printf( "Not found '%s'. Skip.\n", fn1o )
	  printf( "Not found '%s'. Skip.\n", fn1o, >> logfn )
	  next
	}

	if( access( i_fn2//".imh" ) == 1 )
	{
	  imdelete( i_fn2, ver- )
	}

        imarith( fn1o, "*", fullkappa, tmp_fn1,\
                 pixtype="real", calctype="real", ver- )
        imarith( fn1e, "+", tmp_fn1, i_fn2,\
                 pixtype="real", calctype="real", ver- )

        if( access( fullsens//".imh" ) == 1 )
	{
          imarith( i_fn2, "/", fullsens, i_fn2,\
                   pixtype="real", calctype="real", ver- )
	}
	else
	{
	  printf( "Sens func: '%s' not found. Assumed to 1\n", fullsens )
	  printf( "Sens func: '%s' not found. Assumed to 1\n", fullsens,\
                  >> logfn )
        }
        imdelete( tmp_fn1, ver- )

#---- calculation of i-image (binned)

        if( access( i_fn2 ) == 1 )
	  delete( i_fn2, ver- )
        if( access( i_fn1 ) == 1 )
	  delete( i_fn1, ver- )

        if( access( i_fn1//".imh" ) == 1 )
	  imdelete( i_fn1, ver- )

        if( n_version == 210 )
        { 
          wfits( i_fn2//".imh", i_fn2, bscale=1.0, bzero=0.0, long_header-,
                 short_header-, bitpix=-32, scale-, autoscale- )
        }
        else
	{
          if( n_version == 211 )
          {
            wfits( i_fn2//".imh", i_fn2, bscale=1.0, bzero=0.0,\
	           long_header-,
                   fextn="", short_header-, bitpix=-32, scale-, autoscale- )
          }
          else
          {
            printf( "IRAF version error (%d)\n", n_version )
            printf( "IRAF version error (%d)\n", n_version, >> logfn )
            bye
          }
	}
	copy( i_fn2, i_fn1, ver- )
	hbscbin( i_fn2, binfn, etcdir, fnphot, i_fn1, binmode )
	delete( i_fn2, ver- )

        if( n_version == 210 )
        {
          rfits( i_fn1, "", i_fn1, make_image+, long_header-,\
	         short_header-,\
                 datatype="real", blank=0., scale-, oldirafname-, offset=0 )
        }
        if( n_version == 211 )
        {
          rfits( i_fn1, "", i_fn1//".imh", make_image+, long_header-,\
	         short_header-,\
                 datatype="real", blank=0., scale-, oldirafname-, offset=0 )
        }
	delete( i_fn1, ver- )
        hbscoord( i_fn1 )

      }

#---- q-image

      if( access( qlist[i] ) == 0 )
      {
        printf( "Not found '%s'. Skipped.\n", qlist[i] )
        printf( "Not found '%s'. Skipped.\n", qlist[i], >> logfn )
      }
      else
      {
        printf( "---- make q-images ----\n" )
        printf( "---- make q-images ----\n", >> logfn )
        list = qlist[i]
        weightlist = qlist[i]//"w"
        if( access( weightlist ) == 1 )
          delete( weightlist, ver- )
        while( fscan( list, qumode, buf1, buf2, qu_fn ) != EOF )
        {
          if( qumode != "KAPPA" )
  	  {
	    printf( "Error: HBSIQUMAKE failure in '%s'. Check Images",\
	     qlist[i] )
	    printf( "Error: HBSIQUMAKE failure in '%s'. Check Images",\
	     qlist[i], >> logfn )
	    break
	  }
	  if( substr( buf2, 1, 3 ) == "KAP" )
	  {
	    flag_01 = 1.0
            fn1e = buf1//"bn.e"
            fn1o = buf1//"bn.o"
            fn1o_tmp = buf1//"co.o"
	    odall = buf1//"co.e,"//buf1//"co.o"
	    kappa = buf2//kappaext
	    kappas = buf2
            fullkappa = caldir//kappa
            fullkappas = caldir//kappas
	  }
	  else
	  {
	    flag_01 = -1.0
            fn1e = buf2//"bn.e"
            fn1o = buf2//"bn.o"
            fn1o_tmp = buf1//"co.o"
	    odall = buf2//"co.e,"//buf2//"co.o"
	    kappa = buf1//kappaext
	    kappas = buf1
            fullkappa = caldir//kappa
            fullkappas = caldir//kappas
	  } 

          printf( " %s, %s*%s (%4.1f) --->  %s\n", fn1e, fn1o, kappa,\
	          flag_01, qu_fn )
          printf( " %s, %s*%s (%4.1f) --->  %s\n", fn1e, fn1o, kappa,\
	          flag_01, qu_fn, >> logfn )

	  if( access( fn1e//".imh" ) == 0 )
	  {
	    printf( "Not found '%s'. Skip.\n", fn1e )
	    printf( "Not found '%s'. Skip.\n", fn1e, >> logfn )
	    next
 	  }
	  if( access( fn1o_tmp//".imh" ) == 0 )
          {
	    printf( "Not found '%s'. Skip.\n", fn1o_tmp )
	    printf( "Not found '%s'. Skip.\n", fn1o_tmp, >> logfn )
	    next
	  }
	  if( access( fullkappa//".imh" ) == 0 )
	  {
	    if( access( fullkappas//".imh" ) == 0 )
	    {
	      printf( "Not found either '%s' or '%s'. Skip.\n",\
	              fullkappa, fullkappas )
	      printf( "Not found either '%s' or '%s'. Skip.\n",\
	              fullkappa, fullkappas, >> logfn )
	      next
	    }
	    printf( "Substituting %s for %s.\n", kappas, kappa )
	    printf( "Substituting %s for %s.\n", kappas, kappa, >> logfn )
	    fullkappa = fullkappas
	  } 

	  if( access( fn1o//".imh" ) == 1 )
	  {
	    imdelete( fn1o, ver- )
	  }

	  if( access( qu_fn//".imh" ) == 1 )
	  {
	    imdelete( qu_fn, ver- )
          }

  #--- recalculation and binning of odd image

          if( access( fullkappa//".imh" ) == 0 )
	  {
	    printf( "Cannot find %s. Please check calibration files.\n",\
	    fullkappa )
	    printf( "Cannot find %s. Please check calibration files.\n",\
	    fullkappa, >> logfn )
	    return
	  }

          imarith( fn1o_tmp, "*", fullkappa, tmp_fn1,\
                   pixtype="real", calctype="real", ver- )

          if( n_version == 210 )
          { 
            wfits( tmp_fn1//".imh", tmp_fn1, bscale=1.0, bzero=0.0,\
	           long_header-,\
                   short_header-, bitpix=-32, scale-, autoscale- )
          }
          else
          {
            if( n_version == 211 )
            {
              wfits( tmp_fn1//".imh", tmp_fn1, bscale=1.0, bzero=0.0,\
	             long_header-,
                     fextn="", short_header-, bitpix=-32, scale-, autoscale- )
            }
            else
            {
              printf( "IRAF version erorr (%d)\n", n_version )
              printf( "IRAF version erorr (%d)\n", n_version, >> logfn )
              bye
            }
	  }
	  copy( tmp_fn1, fn1o, ver- )
	  imdelete( tmp_fn1, ver- )

	  hbscbin( tmp_fn1, binfn, etcdir, fnphot, fn1o, binmode )

	  delete( tmp_fn1, ver- )

          if( n_version == 210 )
          {
            rfits( fn1o, "", fn1o, make_image+, long_header-,\
	           short_header-,\
                   datatype="real", blank=0., scale-, oldirafname-, offset=0 )
          }
          if( n_version == 211 )
          {
            rfits( fn1o, "", fn1o//".imh", make_image+, long_header-,\
	           short_header-,\
                   datatype="real", blank=0., scale-, oldirafname-, offset=0 )
          }
	  delete( fn1o, ver- )
          hbscoord( fn1o )

  #----- calculation of Stokes parameters: Q/I

          # make weight list
          imsum( odall, tmp_fn1,\
                   pixtype="real", calctype="real", option="sum", ver- )
          imstat( tmp_fn1//"["//x1//":"//x2//"]", fields="midpt",\
                  format- ) | scan( mflux )
          imdelete( tmp_fn1, ver- )
	  printf( "%f\n", ( mflux / ( 1000000.0 * geff ) ), >> weightlist )

          imarith( fn1o, "-", fn1e, tmp_fn2,\
                   pixtype="real", calctype="real", ver- )
          imarith( fn1o, "+", fn1e, tmp_fn3,\
                   pixtype="real", calctype="real", ver- )
          imarith( tmp_fn2, "/", tmp_fn3, qu_fn,\
                   pixtype="real", calctype="real", ver- )
          imdelete( tmp_fn2, ver- )
          imdelete( tmp_fn3, ver- )
          imarith( qu_fn, "*", flag_01, qu_fn,\
                   pixtype="real", calctype="real", ver- )
          printf( "%s ", qu_fn )
          printf( "%s ", qu_fn, >> logfn )
          imstat( qu_fn//"["//x1//":"//x2//"]", fields="mean,stddev", format- )
          imstat( qu_fn//"["//x1//":"//x2//"]", fields="mean,stddev", format-, >> logfn )
        }
      }

#---- u-image

      if( access( ulist[i] ) == 0 )
      {
        printf( "Not found '%s'. Skipped.\n", ulist[i] )
        printf( "Not found '%s'. Skipped.\n", ulist[i], >> logfn )
      }
      else
      {
        printf( "---- make u-images ----\n" )
        printf( "---- make u-images ----\n", >> logfn )
        list = ulist[i]
        weightlist = ulist[i]//"w"
        if( access( weightlist ) == 1 )
	  delete( weightlist, ver- )
        while( fscan( list, qumode, buf1, buf2, qu_fn ) != EOF )
        {
          if( qumode != "KAPPA" )
          {
	    printf( "Error: HBSIQUMAKE failure in '%s'. Check Images", ulist[i] )
	    printf( "Error: HBSIQUMAKE failure in '%s'. Check Images", ulist[i], >> logfn )
	    break
	  }

	  if( substr( buf2, 1, 3 ) == "KAP" )
	  {
	    flag_01 = 1.0
            fn1e = buf1//"bn.e"
            fn1o = buf1//"bn.o"
            fn1o_tmp = buf1//"co.o"
	    odall = buf1//"co.e,"//buf1//"co.o"
	    kappa = buf2//kappaext
	    kappas = buf2
            fullkappa = caldir//kappa
            fullkappas = caldir//kappas
	  }
	  else
	  {
	    flag_01 = -1.0
            fn1e = buf2//"bn.e"
            fn1o = buf2//"bn.o"
            fn1o_tmp = buf1//"co.o"
	    odall = buf2//"co.e,"//buf2//"co.o"
	    kappa = buf1//kappaext
	    kappas = buf1
            fullkappa = caldir//kappa
            fullkappas = caldir//kappas
	  }

          printf( " %s, %s*%s (%4.1f) --->  %s\n", fn1e, fn1o, kappa,\
	          flag_01, qu_fn, >> logfn )

	  if( access( fn1e//".imh" ) == 0 )
	  {
	    printf( "Not found '%s'. Skip.\n", fn1e )
	    printf( "Not found '%s'. Skip.\n", fn1e, >> logfn )
	    next
          }
	  if( access( fn1o_tmp//".imh" ) == 0 )
	  {
	    printf( "Not found '%s'. Skip.\n", fn1o_tmp )
	    printf( "Not found '%s'. Skip.\n", fn1o_tmp, >> logfn )
	    next
	  }
	  if( access( fullkappa//".imh" ) == 0 )
	  {
	    if( access( fullkappas//".imh" ) == 0 )
	    {
	      printf( "Not found either '%s' or '%s'. Skip.\n",\
	              fullkappa, fullkappas )
	      printf( "Not found either '%s' or '%s'. Skip.\n",\
	              fullkappa, fullkappas, >> logfn )
	      next
	    }
	    printf( "Substituting %s for %s.\n", kappas, kappa )
	    printf( "Substituting %s for %s.\n", kappas, kappa, >> logfn )
	    fullkappa = fullkappas
	  }

	  if( access( fn1o//".imh" ) == 1 )
	  {
	    imdelete( fn1o, ver- )
	  }

	  if( access( qu_fn//".imh" ) == 1 )
	  {
	    imdelete( qu_fn, ver- )
	  }

  #--- recalculation and binning of odd image

          if( access( fullkappa//".imh" ) == 0 )
	  {
	    printf( "Cannot find %s. Please check calibration files.\n",\
	    fullkappa )
	    printf( "Cannot find %s. Please check calibration files.\n",\
	    fullkappa, >> logfn )
	    return
	  }

          imarith( fn1o_tmp, "*", fullkappa, tmp_fn1,\
                   pixtype="real", calctype="real", ver- )

          if( n_version == 210 )
          { 
            wfits( tmp_fn1//".imh", tmp_fn1, bscale=1.0, bzero=0.0, long_header-,
                   short_header-, bitpix=-32, scale-, autoscale- )
          }
          else
	  {
            if( n_version == 211 )
            {
              wfits( tmp_fn1//".imh", tmp_fn1, bscale=1.0, bzero=0.0,\
	             long_header-,
                     fextn="", short_header-, bitpix=-32, scale-, autoscale- )
            }
            else
            {
              printf( "IRAF version erorr (%d)\n", n_version )
              printf( "IRAF version erorr (%d)\n", n_version, >> logfn )
              bye
            }
	  }
	  copy( tmp_fn1, fn1o, ver- )
	  imdelete( tmp_fn1, ver- )

	  hbscbin( tmp_fn1, binfn, etcdir, fnphot, fn1o, binmode )

	  delete( tmp_fn1, ver- )

          if( n_version == 210 )
          {
            rfits( fn1o, "", fn1o, make_image+, long_header-,\
	           short_header-,\
                   datatype="real", blank=0., scale-, oldirafname-, offset=0 )
          }
          if( n_version == 211 )
          {
            rfits( fn1o, "", fn1o//".imh", make_image+, long_header-,\
	           short_header-,\
                   datatype="real", blank=0., scale-, oldirafname-, offset=0 )
          }
	  delete( fn1o, ver- )
          hbscoord( fn1o )

  #----- calculation of Stokes parameters: U/I

          # make weight list
          imsum( odall, tmp_fn1,\
                   pixtype="real", calctype="real", option="sum", ver- )
          imstat( tmp_fn1//"["//x1//":"//x2//"]", fields="midpt",\
	          format- ) | scan( mflux )
          imdelete( tmp_fn1, ver- )
	  printf( "%f\n", ( mflux / ( 1000000.0 * geff ) ), >> weightlist )

          imarith( fn1o, "-", fn1e, tmp_fn2,\
                   pixtype="real", calctype="real", ver- )
          imarith( fn1o, "+", fn1e, tmp_fn3,\
                   pixtype="real", calctype="real", ver- )
          imarith( tmp_fn2, "/", tmp_fn3, qu_fn,\
                   pixtype="real", calctype="real", ver- )
          imdelete( tmp_fn2, ver- )
          imdelete( tmp_fn3, ver- )
          imarith( qu_fn, "*", flag_01, qu_fn,\
                   pixtype="real", calctype="real", ver- )
          printf( "%s ", qu_fn )
          printf( "%s ", qu_fn, >> logfn )
          imstat( qu_fn//"["//x1//":"//x2//"]", fields="mean,stddev", format- )
          imstat( qu_fn//"["//x1//":"//x2//"]", fields="mean,stddev", format-, >> logfn )
        }
      }

#---- kappa-image

      printf( "---- make kappa-images ----\n" )
      printf( "---- make kappa-images ----\n", >> logfn )

      if( access( klist0 ) == 1 )
      {
        list = klist0
        weightlist = klist0//"w"
        if( access( weightlist ) == 1 )
          delete( weightlist, ver- )
        while( fscan( list, buf1, kap_fn ) != EOF )
        {
          fn1e = buf1//"bn.e"
          fn1o = buf1//"bn.o"
          odall = buf1//"co.e,"//buf1//"co.o"
          printf( " %s %s  --->  %s\n", fn1e, fn1o, kap_fn )
          printf( " %s %s  --->  %s\n", fn1e, fn1o, kap_fn, >> logfn )

	  if( access( fn1e//".imh" ) == 0 )
	  {
	    printf( "Not found '%s'. Skip.\n", fn1e )
	    printf( "Not found '%s'. Skip.\n", fn1e, >> logfn )
	    next
	  }
	  if( access( fn1o//".imh" ) == 0 )
          {
	    printf( "Not found '%s'. Skip.\n", fn1o )
	    printf( "Not found '%s'. Skip.\n", fn1o, >> logfn )
	    next
	  }
	  if( access( kap_fn//".imh" ) == 1 )
	  {
	    imdelete( kap_fn, ver- )
	  }
          # make weight list
          imsum( odall, tmp_fn1,\
                   pixtype="real", calctype="real", option="sum", ver- )
          imstat( tmp_fn1//"["//x1//":"//x2//"]", fields="midpt",\
	          format- ) | scan( mflux )
          imdelete( tmp_fn1, ver- )
	  printf( "%f\n", ( mflux / ( 1000000.0 * geff ) ), >> weightlist )

          imarith( fn1e, "/", fn1o, kap_fn,\
                   pixtype="real", calctype="real", ver- )
	  imreplace( kap_fn, 1.0, lower=1.0e+5 )
        }
      }
      if( access( klist1 ) == 1 )
      {
        list = klist1
        weightlist = klist1//"w"
        if( access( weightlist ) == 1 )
          delete( weightlist, ver- )
        while( fscan( list, buf1, kap_fn ) != EOF )
        {
          fn1e = buf1//"bn.e"
          fn1o = buf1//"bn.o"
          odall = buf1//"co.e,"//buf1//"co.o"
          printf( " %s %s  --->  %s\n", fn1e, fn1o, kap_fn )
          printf( " %s %s  --->  %s\n", fn1e, fn1o, kap_fn, >> logfn )

	  if( access( fn1e//".imh" ) == 0 )
	  {
	    printf( "Not found '%s'. Skip.\n", fn1e )
	    printf( "Not found '%s'. Skip.\n", fn1e, >> logfn )
	    next
	  }
	  if( access( fn1o//".imh" ) == 0 )
          {
	    printf( "Not found '%s'. Skip.\n", fn1o )
	    printf( "Not found '%s'. Skip.\n", fn1o, >> logfn )
	    next
	  }
	  if( access( kap_fn//".imh" ) == 1 )
	  {
	    imdelete( kap_fn, ver- )
	  }
          # make weight list
          imsum( odall, tmp_fn1,\
                   pixtype="real", calctype="real", option="sum", ver- )
          imstat( tmp_fn1//"["//x1//":"//x2//"]", fields="midpt",\
	          format- ) | scan( mflux )
          imdelete( tmp_fn1, ver- )
	  printf( "%f\n", ( mflux / ( 1000000.0 * geff ) ), >> weightlist )
  
          imarith( fn1e, "/", fn1o, kap_fn,\
                   pixtype="real", calctype="real", ver- )
	  imreplace( kap_fn, 1.0, lower=1.0e+5 )
        }
      }
      if( access( klist2 ) == 1 )
      {
        list = klist2
        weightlist = klist2//"w"
        if( access( weightlist ) == 1 )
          delete( weightlist, ver- )
        while( fscan( list, buf1, kap_fn ) != EOF )
        {
          fn1e = buf1//"bn.e"
          fn1o = buf1//"bn.o"
          odall = buf1//"co.e,"//buf1//"co.o"
          printf( " %s %s  --->  %s\n", fn1e, fn1o, kap_fn )
          printf( " %s %s  --->  %s\n", fn1e, fn1o, kap_fn, >> logfn )

	  if( access( fn1e//".imh" ) == 0 )
	  {
	    printf( "Not found '%s'. Skip.\n", fn1e )
	    printf( "Not found '%s'. Skip.\n", fn1e, >> logfn )
	    next
	  }
	  if( access( fn1o//".imh" ) == 0 )
          {
	    printf( "Not found '%s'. Skip.\n", fn1o )
	    printf( "Not found '%s'. Skip.\n", fn1o, >> logfn )
	    next
	  }
	  if( access( kap_fn//".imh" ) == 1 )
	  {
	    imdelete( kap_fn, ver- )
	  }
          # make weight list
          imsum( odall, tmp_fn1,\
                   pixtype="real", calctype="real", option="sum", ver- )
          imstat( tmp_fn1//"["//x1//":"//x2//"]", fields="midpt",\
	          format- ) | scan( mflux )
          imdelete( tmp_fn1, ver- )
	  printf( "%f\n", ( mflux / ( 1000000.0 * geff ) ), >> weightlist )
  
          imarith( fn1e, "/", fn1o, kap_fn,\
                   pixtype="real", calctype="real", ver- )
	  imreplace( kap_fn, 1.0, lower=1.0e+5 )
        }
      }
      if( access( klist3 ) == 1 )
      {
        list = klist3
        weightlist = klist3//"w"
        if( access( weightlist ) == 1 )
	  delete( weightlist, ver- )
        while( fscan( list, buf1, kap_fn ) != EOF )
        {
          fn1e = buf1//"bn.e"
          fn1o = buf1//"bn.o"
          odall = buf1//"co.e,"//buf1//"co.o"
          printf( " %s %s  --->  %s\n", fn1e, fn1o, kap_fn )
          printf( " %s %s  --->  %s\n", fn1e, fn1o, kap_fn, >> logfn )

	  if( access( fn1e//".imh" ) == 0 )
	  {
	    printf( "Not found '%s'. Skip.\n", fn1e )
	    printf( "Not found '%s'. Skip.\n", fn1e, >> logfn )
	    next
	  }
	  if( access( fn1o//".imh" ) == 0 )
          {
	    printf( "Not found '%s'. Skip.\n", fn1o )
	    printf( "Not found '%s'. Skip.\n", fn1o, >> logfn )
	    next
	  }
	  if( access( kap_fn//".imh" ) == 1 )
	  {
	    imdelete( kap_fn, ver- )
	  }
          # make weight list
          imsum( odall, tmp_fn1,\
                   pixtype="real", calctype="real", option="sum", ver- )
          imstat( tmp_fn1//"["//x1//":"//x2//"]", fields="midpt",\
	           format- ) | scan( mflux )
          imdelete( tmp_fn1, ver- )
	  printf( "%f\n", ( mflux / ( 1000000.0 * geff ) ), >> weightlist )
  
          imarith( fn1e, "/", fn1o, kap_fn,\
                   pixtype="real", calctype="real", ver- )
	  imreplace( kap_fn, 1.0, lower=1.0e+5 )
        }
      }

    }

  }
  

end
