#
#       HBS REDUCTION SUB ROUTINE
#
#     Make unpolarized flat-field image from sky frames of which
#     position angle have differences of 45 degrees
#                             : hbsmakefl2.cl
#
#       1996/12/30      Ver. 1.00    K.Kawabata
#       1996/01/01      Ver. 1.10    K.Kawabata  sequencial interp
#       1998/04/30      Ver. 2.00    K.Kawabata  unification
#
#

procedure hbsmakefl2( inlist1, inlist2, keyitem, logcalc, outframe )

file inlist1	{ prompt = "Image list of a PA1" }
file inlist2	{ prompt = "Image list of a PA2=PA1+45" }
string keyitem	{ prompt = "Refference keyword item" }
int logcalc	{ prompt = "Interpolation by [0]:linear-scale [1]:log-scale" }
file outframe	{ prompt = "Output unpolarized image" }

begin

  string fn1[200], fn2[200]
  real jd1[200], jd2[200]
  string inli1, inli2
  string keyname
  int n1, n2, poshwp
  int last_poshwp = -999
  string outimage
  int n_out = 0

  string buf, buf00, buf45
  int i, j
  int flag_log  # 0:linear 1:log

#----- initial settings

  inli1 = inlist1
  inli2 = inlist2
  keyname = keyitem
  flag_log = logcalc
  outimage = outframe

  buf00 = "hbs.tmp.image.00"
  buf45 = "hbs.tmp.image.45"

  if( access( outimage//".imh" ) == 1 )
    imdelete( outimage, ver- )

  if( access( buf00//".imh" ) == 1 )
    imdelete( buf00, ver- )

  if( access( buf45//".imh" ) == 1 )
    imdelete( buf45, ver- )

  if( flag_log != 0 && flag_log != 1 )
    flag_log = 1

#----- Scanning list file

  n1 = 0
  list = inli1
  while( fscan( list, buf ) != EOF )
  {
    n1 += 1
    fn1[n1] = buf
    imgets( fn1[n1], keyname )
    jd1[n1] = real( imgets.value )
    imgets( fn1[n1], "POS-HWP" )
    poshwp = int( imgets.value )
    if( n1 > 1 && poshwp != last_poshwp )
    {
      printf( "Warning: Position Angle of HWP not equal in '%s'\n", fn1[n1] )
    }
    last_poshwp = poshwp
  }

  n2 = 0
  list = inli2
  while( fscan( list, buf ) != EOF )
  {
    n2 += 1
    fn2[n2] = buf
    imgets( fn2[n2], keyname )
    jd2[n2] = real( imgets.value )
    imgets( fn2[n2], "POS-HWP" )
    poshwp = int( imgets.value )
    if( n2 > 1 && poshwp != last_poshwp )
    {
      printf( "Warning: Position Angle of HWP not equal in '%s'\n", fn2[n2] )
    }
    last_poshwp = poshwp
  }

#----- 0---45---0 ( interpolate 0 )

  for( i=1; i<=(n1-1); i+=1 )
  {

    for( j=1; j<=n2; j+=1 )
    {
      if( ( ( jd2[j] - jd1[i] ) * ( jd2[j] - jd1[i+1] ) ) < 0 )
        goto interp_0
    }
    next
 
 interp_0:

    printf( "%s--%s--%s\n", fn1[i], fn2[j], fn1[i+1] )

    hbsiminterp( fn1[i], jd1[i], fn1[i+1], jd1[i+1], flag_log, buf00, jd2[j] )

    if( n_out == 0 )
    {
      imarith( fn2[j], "+", buf00, outimage, title="", hparams="EXPTIME",\
             pixtype="real", calctype="real", ver- )
      n_out += 1
    }
    else
    {
      imarith( outimage, "+", fn2[j], outimage, title="", hparams="EXPTIME",\
             pixtype="real", calctype="real", ver- )
      imarith( outimage, "+", buf00, outimage, title="", hparams="EXPTIME",\
             pixtype="real", calctype="real", ver- )
      n_out += 1
    }
    imdelete( buf00, ver- )
  }


#----- 45---0---45 ( interpolate 45 )

  for( i=1; i<=(n2-1); i+=1 )
  {
    for( j=1; j<=n1; j+=1 )
    {
      if( ( ( jd1[j] - jd2[i] ) * ( jd1[j] - jd2[i+1] ) ) < 0 )
        goto interp_45
    }
    next
 
 interp_45:

    printf( "%s--%s--%s\n", fn2[i], fn1[j], fn2[i+1] )

    hbsiminterp( fn2[i], jd2[i], fn2[i+1], jd2[i+1], flag_log, buf45, jd1[j] )
    if( n_out == 0 )
    {
      imarith( fn1[j], "+", buf45, outimage, title="", hparams="EXPTIME",\
             pixtype="real", calctype="real", ver- )
      n_out += 1
    }
    else
    {
      imarith( outimage, "+", fn1[j], outimage, title="", hparams="EXPTIME",\
             pixtype="real", calctype="real", ver- )
      imarith( outimage, "+", buf45, outimage, title="", hparams="EXPTIME",\
             pixtype="real", calctype="real", ver- )
      n_out += 1
    }
    imdelete( buf45, ver- )
  }

#----- normalize

  imarith( outimage, "/", n_out, outimage, pixtype="real",\
           calctype="real", ver- )


end
