#
#       HBS REDUCTION SUB ROUTINE
#
#       Pixel sensitivity correction: hbspixsens.cl
#
#       1997/04/09      K.Kawabata  Ver.1.00  read from HBS.list
#       1997/11/14      K.Kawabata  Ver.1.02  overwrite mode
#       1998/04/20      K.Kawabata  Ver.2.00  unification
#       1998/05/31      K.Kawabata  Ver.2.10
#       2000/01/19      K.Kawabata  Ver.3.00
#
procedure hbspixsens

begin

  file fn, fn1, fn2, fnnext
  string fn_stack[500]
  int n_image
  file tmp_list
  int c, i, j
  string hbslist[500]
  string shorttyp[500], title[500]
  string exptime[500], diaph[500], filter[500]
  string slit[500]
  string listoflist, buf
  string ref_dir, caldir
  string fn_2h, fn_2s2
  string buf1, buf2, buf3, buf4, buf5, buf6, buf7
  file logfn
  int n_list
  file fn_pixsens
  int flag_erase # 0:keep old images, 1:erase old images

  int flag_inquiry   # 1: 'flag_overwrite' applys to all image
  int flag_overwrite # 0:skip 1:overwrite
  int flag_ver       # 0:no display 1:display

#--- initial settings

  logfn = envget( "_logfile" )

  if( access( "hbs.tmp.time" ) == 1 )
    delete( "hbs.tmp.time", ver- )
  time >> hbs.tmp.time
  list = "hbs.tmp.time"
  if( fscan( list, buf, buf1, buf2 ) != 3 )
  {
    printf( "Error: time task -- reading\n" )
    bye
  }
  delete( "hbs.tmp.time", ver- )

  print( "" )
  printf( "%s %s\n", buf1, buf2 )
  print( "------------------------------" )
  print( " Pixel sensitivity correction" )
  print( "------------------------------" )
  print( "" )

  print( "", >> logfn )
  printf( "%s %s\n", buf1, buf2, >> logfn )
  print( "------------------------------", >> logfn )
  print( " Pixel sensitivity correction", >> logfn )
  print( "------------------------------", >> logfn )
  print( "", >> logfn )

  listoflist = envget( "_rootlist" )
  caldir = envget( "_caldir" )
  fn_2h = envget( "_fl2hlp" )
  fn_2s2 = envget( "_fl2s2p" )

  tmp_list = mktemp( "hbs.tmp.list." )

  if( access( listoflist ) == 0 )
  {
    print( "Cannot find "//listoflist//"   Please do hbsgroup" )
    print( "Cannot find "//listoflist//"   Please do hbsgroup", >> logfn )
    bye
  }

#--- Verbose Mode

  if( envget( "_fg_ver" ) == "1" )
  {
    flag_ver = 1
  }
  else
  {
    if( envget( "_fg_ver" ) == "2" )
    {
      flag_ver = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n" )
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n", >> logfn )
      bye
    }
  }

#----- Replace Mode

  if( envget( "_fg_mode" ) == "1" )
  {
    flag_inquiry = 1
    flag_overwrite = 1
  }
  else
  {
    if( envget( "_fg_mode" ) == "2" )
    {
      flag_inquiry = 1
      flag_overwrite = 0
    }
    else
    {
      if( envget( "_fg_mode" ) == "3" )
      {
        flag_inquiry = 0
        flag_overwrite = 0
      }
      else
      {
        printf( "'hbsinit' process is incomplete.(_fg_mode) Retry.\n" )
        printf( "'hbsinit' process is incomplete.(_fg_mode) Retry.\n",\
                >> logfn )
        bye
      }
    }
  }
    
#----- Erase/Reserve old image

  if( envget( "_fg_del1" ) == "1" )
  {
    flag_erase = 1
  }
  else
  {
    if( envget( "_fg_del1" ) == "2" )
    {
      flag_erase = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_del1) Retry.\n" )
      printf( "'hbsinit' process is incomplete.(_fg_del1) Retry.\n", >> logfn )
      bye
    }
  }

#----- Scanning the list file

  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "HBS" )
    {
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL" )
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL",\
             >> logfn )
      break
    }
    n_list += 1
    hbslist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#----- correction of pixel sensitivity

  print( "Now correcting sensitivity of each pixel...  *.tr.imh -> *.px.imh" )
  print( "Now correcting sensitivity of each pixel...  *.tr.imh -> *.px.imh",\
         >> logfn )
  for( i=1; i<=n_list; i+=1 )
  {
    if( shorttyp[i] != "DAR" && shorttyp[i] != "BIA" &&\
        shorttyp[i] != "NEA" )
    {
      ref_dir = "none"

      if( substr( diaph[i], 1, 3 ) == "2HL" )
      {
        ref_dir = caldir
        fn_pixsens = fn_2h
      }
      if( substr( diaph[i], 1, 3 ) == "2S2" )
      {
        ref_dir = caldir
        fn_pixsens = fn_2s2
      }

      if( ref_dir == "none" ){
        printf( "Warning: Unknown Diaph type in %s (%s)\n",\
	        hbslist[i], diaph[i] )
        printf( "Warning: Unknown Diaph type in %s (%s)\n",\
	        hbslist[i], diaph[i], >> logfn )
	next
      }

      if( flag_ver == 1 )
      {
        printf( "--- OBJECT LIST : %s ---\n", hbslist[i] )
        printf( "  refereing calib's in %s\n", ref_dir )
      }
      printf( "--- OBJECT LIST : %s ---\n", hbslist[i], >> logfn )
      printf( "  refereing calib's in %s\n", ref_dir, >> logfn )

      sed( 's/fit\.imh//', hbslist[i], > tmp_list )

      n_image = 0
      list = tmp_list
      while( fscan( list, buf ) != EOF )
      {
        n_image += 1
	fn_stack[n_image] = buf
      }
      delete( tmp_list, ver- )

      for( j=1; j<=n_image; j+=1 )
      {
        fn = fn_stack[j]

        fn1 = fn//"tr"
        fn2 = fn//"px"

	if( j < n_image )
         fnnext = fn_stack[j+1]//"px"
	else
         fnnext = "none"

        if( access( fn1//".imh" ) == 0 ){
          if( flag_ver == 1 )
            printf( "%s--(not exist. skip.)  ", fn1 )
          printf( "%s--(not exist. skip.)  ", fn1, >> logfn )
          next
        }

        if( access( fn2//".imh" ) == 1 ){

          if( flag_overwrite == 0 )
          {
            if( fnnext != "none" && access( fnnext//".imh" ) == 0 )
            {
	      printf( "\n%s: Maybe insufficient proccess last time.\n", fn2 )
              goto inquiry
            }

            if( flag_inquiry == 1 )
            {
              if( flag_ver == 1 )
                printf( "%s--(already exists. skip.)  ", fn2 )
              printf( "%s--(already exists. skip.)  ", fn2, >> logfn )
              next
            }
            else
	    {
              printf( "\n%s already exist.\n", fn2 )
              printf( "\n%s already exist.\n", fn2, >> logfn )
          inquiry:
              printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> " )
              printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> ", >> logfn )
              c = scan( buf )
              while( buf != "Y" && buf != "y" && buf != "n" && buf != "N" )
                c = scan( buf )
              printf( "%s\n", buf, >> logfn )
              if( buf == "Y" )
              {
                flag_inquiry = 1
                flag_overwrite = 1
              }
              if( buf == "N" )
              {
                flag_inquiry = 1
                flag_overwrite = 0
                if( flag_ver == 1 )
                  printf( "%s--(already exists. skip.)  ", fn2 )
                printf( "%s--(already exists. skip.)  ", fn2, >> logfn )
                next
              }
              if( buf == "n" )
              {
                if( flag_ver == 1 )
                  printf( "%s--(already exists. skip.)  ", fn2 )
                printf( "%s--(already exists. skip.)  ", fn2, >> logfn )
                next
              }
            }
          }
          if( flag_ver == 1 )
            printf( "%s(overwrite)  ", fn2 )
          printf( "%s(overwrite)  ", fn2, >> logfn )
          imdelete( fn2, ver- )
        }
        else
        {
          if( flag_ver == 1 )
            printf( "%s  ", fn2 )
          printf( "%s  ", fn2, >> logfn )
        }

        imarith( fn1, "/", ref_dir//fn_pixsens, fn2, divzero=0.0,\
                 hparams="", pixtype="real", calctype="real", ver- )

	if( access( fn2//".imh" ) == 0 )
	{
	  printf( "Pix-sens-correction Failed. Abnormal end.\n" )
	  printf( "Pix-sens-correction Failed. Abnormal end.\n", >> logfn )
	  bye
	}

        hedit( fn2, 'HISTORY', "Pixel-Sensitivity Corrected using "\
	       //ref_dir//fn_pixsens, add+, del-, ver-, show-,\
	       update+, mode=mode )

	if( flag_erase == 1 )
          imdelete( fn1, ver- )
      }
      if( flag_ver == 1 )
        printf( "\n" )
      printf( "\n", >> logfn )
    }
  }

end
