#
#       HBS REDUCTION SUB ROUTINE
#
#      Prescan and make template four spectra : hbsprescan.cl
#
#       1997/03/21      K.Kawabata  Ver.0.10  for CRYOCAM manual reduc
#       1997/04/10      K.Kawabata  Ver.0.20  refer HBS.list
#       1997/06/30      K.Kawabata  Ver.0.30  for original fitting
#


procedure hbsprescan


begin

  file listoflist = "HBS.list"
  file fn_scan = "prescan.px"
  file fn_y4 = "hbsprescan.dat"
  string hbslist[50]
  string shorttyp[50], title[50]
  string exptime[50], diaph[50], filter[50]
  string slit[50]
  file tmp_fn, tmp_list, tmp_tbl1
  int c, i, j, n_list
  int nx, ny
  string buf = ""
  string buf1, buf2, buf3, buf4, buf5, buf6, buf7
  real slicelevel = 0.1
  real dx = 10
  real y[4], y1, y2, y3

  tmp_fn = mktemp( "hbs.tmp.image." )
  tmp_list = mktemp( "hbs.tmp.list." )
  tmp_tbl1 = mktemp( "hbs.tmp.fits." )


  print( "" )
  print( "-----------------------------------------------------" )
  print( " make a template of distortion scanning four spectra" )
  print( "-----------------------------------------------------" )
  print( "" )


#---- confirmation

  if( access( fn_scan//".imh" ) )
  {
    print( "Detected old '"//fn_scan//"'. Use this ? (y/n)  >> " )
    c = scan( buf )
    while( buf != "y" && buf != "n" )
      c = scan( buf )
    if( buf == "y" )
      goto prescanning
    imdelete( fn_scan, ver- )
  }

#----- Scanning the list file

  if( access( listoflist ) == 0 )
  {
    print( "Cannot find "//listoflist//"   Please do hbsgroup" )
    bye
  }
  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "HBS" )
    {
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL" )
      break
    }
    if( buf2 != "OBJ" || buf3 == "SKY" || buf3 == "sky" )
      next
    n_list += 1
    hbslist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#---- selection of images for average

  for( i=1; i<=n_list; i+=1 )
  {
    printf( "%2d: %16s  %6s  %s  %s\n", i, hbslist[i], title[i],\
            diaph[i], filter[i] )
  }

nextstep1:
  printf( "Choose an UPPER-HOLE group  >> " )
  while( scan( buf ) != 1 ){}
  i = int( buf )
  if( i < 1 || i > n_list )
    goto nextstep1
  c = strlen( diaph[i] )
  if( substr( diaph[i], (c-4), c ) != "UPPER" )
  {
    printf( "%s is not using UPPER HOLE !!  avoided.\n", hbslist[i] )
    goto nextstep1
  }
  sed( 's/\.fit\.imh/\.px\.imh/', hbslist[i], >> tmp_list )
  printf( "OK. %s is registed.  More ? [y/n]  >> ", hbslist[i] )
  i = scan( buf )
  while( buf != "y" && buf != "n" )
    i = scan( buf )
  if( buf == "y" )
    goto nextstep1

nextstep2:
  printf( "Choose a LOWER-HOLE group  >> " )
  while( scan( buf ) != 1 ){}
  i = int( buf )
  if( i < 1 || i > n_list )
    goto nextstep2
  c = strlen( diaph[i] )
  if( substr( diaph[i], (c-4), c ) != "LOWER" )
  {
    printf( "%s is not using LOWER HOLE !!  avoided.\n", hbslist[i] )
    goto nextstep2
  }
  sed( 's/\.fit\.imh/\.px\.imh/', hbslist[i], >> tmp_list )
  printf( "OK. %s is registed.  More ? [y/n]  >> ", hbslist[i] )
  i = scan( buf )
  while( buf != "y" && buf != "n" )
    i = scan( buf )
  if( buf == "y" )
    goto nextstep2

#---- make reference image

  imcombine( "@"//tmp_list, fn_scan, combine="average",\
                 reject="none", project-, outtype="real",
                 offsets="none", masktype="none", scale="none",\
                 zero="none", weight="none", lthreshold=INDEF,\
                 hthreshold=INDEF, nlow=1, nhigh=1 )
  delete( tmp_list, ver- )

prescanning:

  boxcar( fn_scan, tmp_fn, dx, 1, boundary="nearest" )

#---- search four spectra and define initial constants

  if( access( "database/id"//fn_scan ) )
      delete( "database/id"//fn_scan )
  if( access( fn_y4 ) )
      delete( fn_y4 )

  print( "mark the 1, 2, 3, 4 peaks in order with [m]-key" )

  identify( fn_scan, section="middle column", databas='database',\
            coordli='home$etc/null.dat', nsum=500, match=20.,\
            maxfeat=4, fwidth=15., cradius=15., functio='legendre',\
            order=2, niterate=10, low_rej=3., high_rej=3.,\
            grow=0.,ftype='emission', mode='ql',thresho=5,\
            autowrite+ )

  list = "database/id"//fn_scan

  for( i=1; i<=4; i+=1 )
  {
    while( fscan( list, buf1, buf, buf, buf, buf, buf ) != 6 ){}
    y[i] = real( buf1 )
    printf( "%6.2f\n", y[i], >> fn_y4 )
  }
  delete( "database/id"//fn_scan )

#---- scan four spectra

  imgets( fn_scan, 'i_naxis1' )
  nx = int( imgets.value )
  imgets( fn_scan, 'i_naxis2' )
  ny = int( imgets.value )

  if( access( tmp_tbl1 ) == 1 )
    delete( tmp_tbl1, ver- )

  for( j=1; j<=4; j+=1 )
  {
     if( access( "hbsprescan."//j ) == 1 )
       delete( "hbsprescan."//j, ver- )
  }

  wfits( tmp_fn, tmp_tbl1, bscale=1.0, bzero=0.0, long_header-,
         short_header-, bitpix=-32, scale-, autoscale- )

  hbsscan4( tmp_tbl1, y[1], y[2], y[3], y[4], slicelevel, 80, nx )

  delete( tmp_tbl1, ver- )

  imdelete( tmp_fn, ver- )
  
end
