#
#       HBS REDUCTION SUB ROUTINE
#
#       Rescaling wavelength for 1d spectra : hbsrescale.cl
#
#       1998/03/07      K.Kawabata  Ver.0.10  for CRYOCAM manual reduc
#       1998/04/20      K.Kawabata  Ver.2.00  unification
#       1998/05/31      K.Kawabata  Ver.2.10
#       1999/06/08      K.Kawabata  Ver.2.20
#

procedure hbsrescale

begin

  file listoflist
  string hbslist[250]
  string shorttyp[250], title[250]
  string exptime[250], diaph[250], filter[250]
  string slit[250]
  int c, i, j, n, n_list

  file fn
  string fn_stack[500]
  int n_image
  string fnobe1, fnobo1, fnobe, fnobo
  file tmp_list
  string buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7
  file logfn
  file okfile = "hbs.tmp.hbscrescale"

  file caldir, fn_def, etcdir
  real crpix, crval, cdelt

  int flag_ver       # 0:no display 1:display
  int n_version      # IRAF version (210 or 211)

#--- initial settings

  logfn = envget( "_logfile" )

  if( access( "hbs.tmp.time" ) == 1 )
    delete( "hbs.tmp.time", ver- )
  time >> hbs.tmp.time
  list = "hbs.tmp.time"
  if( fscan( list, buf, buf1, buf2 ) != 3 )
  {
    printf( "Error: time task -- reading\n" )
    bye
  }
  delete( "hbs.tmp.time", ver- )

  print( "" )
  printf( "%s %s\n", buf1, buf2 )
  print( "--------------------" )
  print( " Rescale wavelength" )
  print( "--------------------" )
  print( "" )

  print( "", >> logfn )
  printf( "%s %s\n", buf1, buf2, >> logfn )
  print( "--------------------", >> logfn )
  print( " Rescale wavelength", >> logfn )
  print( "--------------------", >> logfn )
  print( "", >> logfn )

  listoflist = envget( "_rootlist" )
  caldir = envget( "_caldir" )
  etcdir = envget( "_etcdir" )
  fn_def = caldir//envget( "_defhname" )

  tmp_list = mktemp( "hbs.tmp.list." )

#----- read difinition file

  buf1 = ""
  buf2 = ""
  buf3 = ""

  if( access( fn_def ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_def )
    printf( "Cannot find '%s'\n", fn_def, >> logfn )
    bye
  }

  crpix = 1.0
  crval = 0.0
  cdelt = 0.0

  list = fn_def
  while( fscan( list, buf1, buf2, buf3 ) != EOF )
  {
    if( buf2 == "SW" )
    {
      crval = real( buf3 )
    }
    if( buf2 == "DW" )
    {
      cdelt = real( buf3 )
    }
  }
  if( ( crval * cdelt ) == 0  )
  {
    printf( "Read error in '%s'\n", fn_def )
    printf( "Read error in '%s'\n", fn_def, >> logfn )
    bye
  }

#--- Verbose Mode

  if( envget( "_fg_ver" ) == "1" )
  {
    flag_ver = 1
  }
  else
  {
    if( envget( "_fg_ver" ) == "2" )
    {
      flag_ver = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n" )
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n", >> logfn )
      bye
    }
  }

#----- IRAF version check

  buf1 = envget( "version" )
  c = stridx( "V", buf1 )
  buf2 = substr( buf1, c, ( c + 4 ) )

  n_version = 0

  if( buf2 == "V2.11" )
  {
    n_version = 211
  }
  else
  {
    if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'hbsred'\n" )
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n" )
    printf( "IRAF version check failed.\n", >> logfn )
    printf( "Please input like next line, and restart 'hbsred'\n", >> logfn )
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n", >> logfn )
    bye
  }

#----- Scanning the list file

  if( access( listoflist ) == 0 )
  {
    print( "Cannot find "//listoflist//".   Please do hbsgroup." )
    print( "Cannot find "//listoflist//".   Please do hbsgroup.", >> logfn )
    return
  }

  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "HBS" )
    {
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL" )
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL",\
             >> logfn )
      break
    }
    if( buf2 != "OBJ" && buf2 != "COM" )
      next
    n_list += 1
    hbslist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#----- main routine

mainroutine:

  for( i=1; i<=n_list; i+=1 )
  {
    if( shorttyp[i] != "DAR" && shorttyp[i] != "BIA" &&\
        shorttyp[i] != "NEA" )
    {
      printf( "%3d : %s\n", i, hbslist[i] )
      printf( "%3d : %s\n", i, hbslist[i], >> logfn )
    }
  }
  printf( "--------------\n" )
  printf( "Which is to be processed >> " )
  printf( "--------------\n", >> logfn )
  printf( "Which is to be processed >> ", >> logfn )

inputgroup:
  c = scan( i )
  while( i < 0 || i > n_list )
    c = scan( i )
  if( shorttyp[i] == "DAR" || shorttyp[i] == "BIA" || shorttyp[i] == "NEA" )
    goto inputgroup

  printf( "%d : %s ", i, hbslist[i], >> logfn )

  sed( 's/fit\.imh//', hbslist[i], > tmp_list )

  n_image = 0
  list = tmp_list
  while( fscan( list, buf ) != EOF )
  {
    n_image += 1
    fn_stack[n_image] = buf
  }
  delete( tmp_list, ver- )

  for( j=1; j<=n_image; j+=1 )
  {
    fn = fn_stack[j]

    fnobe1 = fn//"od.e.org"
    fnobo1 = fn//"od.o.org"
    fnobe = fn//"od.e"
    fnobo = fn//"od.o"

    if( access( fnobe//".imh" ) == 0 && access( fnobe1//".imh" ) == 0 )
    {
      printf( "%s--(not exist. skip.)  ", fn )
      printf( "%s--(not exist. skip.)  ", fn, >> logfn )
      next
    }
    if( access( fnobo//".imh" ) == 0 && access( fnobo1//".imh" ) == 0 )
    {
      printf( "%s--(not exist. skip.)  ", fn )
      printf( "%s--(not exist. skip.)  ", fn, >> logfn )
      next
    }
    
    if( access( fnobe1//".imh" ) == 0 )
      imcopy( fnobe, fnobe1, ver- )
    if( access( fnobo1//".imh" ) == 0 )
      imcopy( fnobo, fnobo1, ver- )
      
    if( flag_ver == 1 )
      printf( "%s  ", fn )
    printf( "%s  ", fn, >> logfn )

#---- binning

    imreplace( fnobe1, 0.0, lower=INDEF, upper=0.0 )
    imreplace( fnobo1, 0.0, lower=INDEF, upper=0.0 )

    if( access( fnobe1 ) == 1 )
      delete( fnobe1, ver- )
    if( access( fnobo1 ) == 1 )
      delete( fnobo1, ver- )
    if( access( fnobe ) == 1 )
      delete( fnobe, ver- )
    if( access( fnobo ) == 1 )
      delete( fnobo, ver- )

    if( access( fnobe//".imh" ) == 1 )
      imdelete( fnobe, ver- )
    if( access( fnobo//".imh" ) == 1 )
      imdelete( fnobo, ver- )

    if( n_version == 210 )
    { 
      wfits( fnobe1//".imh", fnobe1, bscale=1.0, bzero=0.0, long_header-,
             short_header-, bitpix=-32, scale-, autoscale- )
      wfits( fnobo1//".imh", fnobo1, bscale=1.0, bzero=0.0, long_header-,
             short_header-, bitpix=-32, scale-, autoscale- )
    }
    else
    {
      if( n_version == 211 )
      {
        wfits( fnobe1//".imh", fnobe1, bscale=1.0, bzero=0.0, long_header-,
               fextn="", short_header-, bitpix=-32, scale-, autoscale- )
        wfits( fnobo1//".imh", fnobo1, bscale=1.0, bzero=0.0, long_header-,
               fextn="", short_header-, bitpix=-32, scale-, autoscale- )
      }
      else
      {
        printf( "IRAF version erorr (%d)\n", n_version )
        printf( "IRAF version erorr (%d)\n", n_version, >> logfn )
        bye
      }
    }
    copy( fnobe1, fnobe, ver- )
    copy( fnobo1, fnobo, ver- )

    hbscrescale( fnobe1, fnobe )
    if( access( okfile ) == 0 )
    {
      delete( fnobe, ver- )
      delete( fnobo, ver- )
      delete( fnobe1, ver- )
      delete( fnobo1, ver- )
      delete( binfn1, ver- )
      next
    }
    hbscrescale( fnobo1, fnobo )
    if( access( okfile ) == 0 )
    {
      delete( fnobe, ver- )
      delete( fnobo, ver- )
      delete( fnobe1, ver- )
      delete( fnobo1, ver- )
      delete( binfn1, ver- )
      next
    }

    delete( okfile, ver- )
    delete( fnobe1, ver- )
    delete( fnobo1, ver- )

    if( n_version == 210 )
    {
      rfits( fnobo, "", fnobo, make_image+, long_header-, short_header-,\
             datatype="real", blank=0., scale-, oldirafname-, offset=0 )
      rfits( fnobe, "", fnobe, make_image+, long_header-, short_header-,\
             datatype="real", blank=0., scale-, oldirafname-, offset=0 )
    }
    if( n_version == 211 )
    {
      rfits( fnobo, "", fnobo//".imh", make_image+, long_header-,\
             short_header-,\
             datatype="real", blank=0., scale-, oldirafname-, offset=0 )
      rfits( fnobe, "", fnobe//".imh", make_image+, long_header-,\
	     short_header-,\
             datatype="real", blank=0., scale-, oldirafname-, offset=0 )
    }
    delete( fnobe, ver- )
    delete( fnobo, ver- )

    imreplace( fnobe, 0.0, lower=INDEF, upper=0.0 )
    hbscoord( fnobe )
    imreplace( fnobo, 0.0, lower=INDEF, upper=0.0 )
    hbscoord( fnobo )

#---- FITS header comments

    hedit( fnobe, "CRPIX1", crpix, add+, delete-, verify-, show-,\
           update+ )
    hedit( fnobe, "CRVAL1", crval, add+, delete-, verify-, show-,\
          update+ )
    hedit( fnobe, "CD1_1", cdelt, add+, delete-, verify-, show-,\
           update+ )
    hedit( fnobe, "CDELT1", cdelt, add+, delete-, verify-, show-,\
           update+ )
    
    hedit( fnobo, "CRPIX1", crpix, add+, delete-, verify-, show-,\
           update+ )
    hedit( fnobo, "CRVAL1", crval, add+, delete-, verify-, show-,\
          update+ )
    hedit( fnobo, "CD1_1", cdelt, add+, delete-, verify-, show-,\
           update+ )
    hedit( fnobo, "CDELT1", cdelt, add+, delete-, verify-, show-,\
           update+ )
  }
  if( flag_ver == 1 )
    printf( "\n" )
  printf( "\n", >> logfn )

end
